# Global variables for all Makefiles in the IDX-PKI hierarchy.
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

.PHONY: all build install test clean uninstall

IDXPKI_TOPDIR:=$(shell cd $(IDXPKI_TOPDIR) && /bin/pwd)

# Main directory for the sources.
IDXPKI_SRCDIR=$(IDXPKI_TOPDIR)/src

# Directory where compiled data is put. This may become a "volatile" option
# in a future version of IDX-PKI, to accomodate variant builds.
IDXPKI_BUILDDIR = $(IDXPKI_TOPDIR)/build

# These ones are not supposed to change but *nothing* may be hardcoded.
IDXPKI_BUILDPERLDIR        = $(IDXPKI_BUILDDIR)/perllib
IDXPKI_BUILDPHPDIR         = $(IDXPKI_BUILDDIR)/phplib
IDXPKI_BUILDSHDIR          = $(IDXPKI_BUILDDIR)/shelllib
IDXPKI_BUILDSBINDIR        = $(IDXPKI_BUILDDIR)/sbin
IDXPKI_BUILDBOOTSTRAPDIR   = $(IDXPKI_BUILDDIR)/bootstrap
IDXPKI_BUILDHTDOCSDIR      = $(IDXPKI_BUILDDIR)/htdocs
IDXPKI_BUILDSYSVDIR        = $(IDXPKI_BUILDDIR)/sysv
IDXPKI_BUILDCONFIGDIR      = $(IDXPKI_BUILDDIR)/config
IDXPKI_BUILDVARDIR         = $(IDXPKI_BUILDDIR)/var
IDXPKI_BUILDLOGSDIR        = $(IDXPKI_BUILDDIR)/logs
IDXPKI_BUILDSECUREDIR      = $(IDXPKI_BUILDDIR)/secure
IDXPKI_BUILDCOMPILEDIR     = $(IDXPKI_BUILDDIR)/compile
IDXPKI_BUILDTESTDIR        = $(IDXPKI_BUILDDIR)/test

# Ditto.
IDXPKI_TOOLSDIR = $(IDXPKI_TOPDIR)/tools
IDXPKI_KEYDIR   = $(IDXPKI_TOPDIR)/src/keys
IDXPKI_ETCDIR   = $(IDXPKI_TOPDIR)/src/etc
IDXPKI_LIBDIR   = $(IDXPKI_TOPDIR)/src/lib

# Directory where the source code of the tests resides
IDXPKI_TESTDIR    = $(IDXPKI_TOPDIR)/src/test
# Directory where intermediate state for tests is stored
IDXPKI_VARTESTDIR = $(IDXPKI_BUILDDIR)/test

include $(IDXPKI_TOPDIR)/config.status

$(IDXPKI_TOPDIR)/config.status:
$(IDXPKI_TOPDIR)/config.status:
	@echo
	@echo
	@echo "********************  PRE-INSTALLATION CONFIGURATION  ********************"
	@echo
	@echo 'Please run "perl ./configure.pl" or "perl ./configure-auto.pl" first'
	@echo '(in directory $(IDXPKI_TOPDIR)).'
	@echo
	@echo "Then you can run $(MAKE) again."
	@echo
	@echo "**************************************************************************"
	@echo
	@echo
	exit 1

######################## Canned rules for build ###########################

# This stuff is there so that a kind of "make export" also happens at
# build time; this way, making "build" from a CVS checkout still
# produces appropriately copyrighted outputs.
copyrightprog = $(IDXPKI_TOOLSDIR)/interne-exporte-code.pl
ifeq (y,$(shell test -f $(copyrightprog) && echo y))
COPYRIGHT=perl $(copyrightprog)
else
COPYRIGHT=cat
endif

# This rule prepends the relevant library paths to a Perl standalone
# script, so that it may find the perllibs with a standard "use" directive.

define ADD_LIBS_TO_PERL_SCRIPT
install -d $(dir $@)
head -1 $< > $@
echo "use lib \"$(CONFIG_PERLLIBDIR)\";" >> $@
echo "use lib \"$(CONFIG_PERLLIBDIR)/i386-linux\";" >> $@
echo >> $@
tail +2 $< | $(COPYRIGHT) >> $@
chmod 755 $@
endef

# Same thing for shell scripts. The variable CONFIG_SHELLLIBDIR is set
# at the beginning of the script, so that one can say for example:
#     . $(CONFIG_SHELLLIBDIR)/CompileTime.sh

define ADD_LIBS_TO_SHELL_SCRIPT
install -d $(dir $@)
head -1 $< > $@
echo 'CONFIG_SHELLLIBDIR="$(CONFIG_SHELLLIBDIR)"' >> $@
echo >> $@
tail +2 $< | $(COPYRIGHT) >> $@
chmod 755 $@
endef

define SIMPLE_COPY
install -d $(dir $@)
$(COPYRIGHT) < $< > $@
endef

define COPY_EXECUTABLE
install -d $(dir $@)
$(COPYRIGHT) < $< > $@
chmod 755 $@
endef
