#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<configure-auto.pl> - IDX-PKI automatic configuration script, which generates F<config.status> for a test (unprivileged) installation

=head1 SYNOPSIS

B<configure-auto.pl>

=head1 DESCRIPTION

This script uses B<configure.pl> to automatically configure a test IDX-PKI
installation.

=cut "

use strict;
use FindBin;
use IO::Pipe;

use vars qw(@test_config_vars @configure_script $config_status);

# This is copy-pasted from IDX::Test...
sub port_range ($) {
	my ($n)=@_;

	my $netstat=new IO::Pipe;
	$netstat->reader("/bin/netstat","-ant") ||
	  die "Can't fork a netstat command: $!\n";

	my %openports;
	while(<$netstat>) {
		next unless (m/LISTEN$/);
		my (undef,undef,undef,$local,undef,undef)=split m/\s/;
		my ($address,$port)=split m/:/,$local;
		$openports{$port}=1;
	}
	$netstat->close();

	my $baseport;
	TRY:do {
		$baseport=int(rand(40000)+4000);
		for(my $i=0; $i<$n; $i++) {
			redo TRY if ($openports{$baseport+$i});
		};
	};
	return $baseport;
}

my $prefix = "$FindBin::RealBin/install_image";
# Range of 20 consecutive free ports for all the needs of the PKI.
my $baseport = port_range (20);

@test_config_vars = (
	'--batch',
	# Installation Options
	'--install-all',
	'--test',
	'--test-privs',

	# Directory and File Names
	'--prefix'                => $prefix,

	# Network Addresses and Ports
	'--port-https'            => ($baseport ++),
	'--port-http'             => ($baseport ++),
	'--port-ldap-repository'  => ($baseport ++),
);

@configure_script = ("perl", "./configure.pl");
$config_status = "./config.status";

rename ($config_status, $config_status.".old") if (-e $config_status);
system (@configure_script, @test_config_vars, @ARGV) &&
	die "Error generating the config.status file (see error messages ".
	"above, if any).";
