#!/bin/sh
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: installrcd.sh,v 1.6 2003/07/23 13:20:42 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/etc/installrcd.sh,v $

# This is a wrapper to update the /etc/rc*.d start and stop links to
# /etc/init.d/<script>, because the functions to do so are not the same
# under RedHat and Debian (*sigh*).

TOBEDONE=$1
SCRIPT=$2

# This can't be used in a non-root installation.
[ `id -u` = 0 ] || { echo "This script must be run as root." >&2; exit 1; }

usage() {
	echo "Usage: installrcd.sh [--install|--remove] <script>"
	exit 1
}

installrcd() {
	if [ -n "/sbin/chkconfig" -a -x "/sbin/chkconfig" ]; then
		# RedHat type.
		/sbin/chkconfig --add "$*"
	elif [ -n "`which update-rc.d 2>/dev/null`" -a \
		-x "`which update-rc.d 2>/dev/null`" ]; then
		# Debian type.
		update-rc.d "$*" defaults
	else
		# No known automatic script.
		for i in 0 1 6; do
			ln -s ../init.d/"$*" /etc/rc$i.d/K60"$*"
		done
		for i in 2 3 4 5; do
			ln -s ../init.d/"$*" /etc/rc$i.d/S90"$*"
		done
	fi
}

uninstallrcd() {
	if [ -n "/sbin/chkconfig" -a -x "/sbin/chkconfig" ]; then
		# RedHat type.
		/sbin/chkconfig --del "$*"
	elif [ -n "`which update-rc.d 2>/dev/null`" -a \
		-x "`which update-rc.d 2>/dev/null`" ]; then
		# Debian type.
		update-rc.d -f "$*" remove
	else
		# No known automatic script.
		for i in 0 1 6; do
			rm -f /etc/rc$i.d/K[0-9][0-9]"$*"
		done
		for i in 2 3 4 5; do
			rm -f /etc/rc$i.d/S[0-9][0-9]"$*"
		done
	fi
}

if [ -z $SCRIPT ]; then
	usage
fi

case $TOBEDONE in
	--install)
		installrcd $SCRIPT || echo "Warning, impossible to install init scripts."
	;;
	--remove)
		uninstallrcd $SCRIPT || echo "Warning, impossible to uninstall init scripts."
	;;
	*)
		usage
	;;
esac

# Success whatever happened.
exit 0
