#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<mkinitd-httpd.pl> - Generates an Apache SysV startup script

=head1 SYNOPSIS

B<mkinitd-httpd.pl>
B<--binary> I<binary>
[B<--port-https> I<port-https>]
[B<--port-http> I<port-http>]
[B<--configdir> I<configdir>]
B<--configfile> I<configfile>
B<--pidfile> I<pidfile>
B<--rootdir> I<rootdir>
[B<--noprivs>]

=head1 DESCRIPTION

Generates an Apache SysV startup script and writes it on the standard
output.  I<binary> is the Apache binary to be used (it may be patched to
allow non-root execution).  I<port-https> and I<port-http> are used for an
informational message.  I<configfile>, I<pidfile>, I<rootdir> and
I<configdir> are respectively the configuration directory, Apache
configuration file, the file containing its PID and its root directory.

If B<--noprivs> is set, the server will be lauched as the current user,
otherwise root must run it.

Some of the default values are taken from the configuration.  If the file
F<config.status> doesn't exist at the top directory or values are missing,
they must be provided on the command line, except for C<--cacert> which is
optional.

=cut "

use strict;

use Getopt::Long;
use Pod::Usage;
use FindBin;

# Include installation tools.
use vars qw ($topdir);
$topdir = "$FindBin::RealBin/../../";
push (@INC, "$topdir/tools");
require InstallLib;
import  InstallLib ();

# Program global variables.
use vars qw (%CONFIG
	$binary
	$porthttps
	$porthttp
	$configdir
	$configfile
	$pidfile
	$rootdir
	$noprivs
	$crlpath
);
%CONFIG = parseConfigStatus ($topdir);
$porthttps      = $CONFIG{'CONFIG_PORT_HTTPS'};
$porthttp       = $CONFIG{'CONFIG_PORT_HTTP'};
$configdir      = $CONFIG{'CONFIG_CONFIGDIR'};
$noprivs        = $CONFIG{'CONFIG_TEST_PRIVS'};
$crlpath		= $CONFIG{'CONFIG_CRLDIR'};

GetOptions (
	'binary=s'     => \$binary,
	'port-https=i' => \$porthttps,
	'port-http=i'  => \$porthttp,
	'configdir=s'  => \$configdir,
	'configfile=s' => \$configfile,
	'pidfile=s'    => \$pidfile,
	'rootdir=s'    => \$rootdir,
	'noprivs!'        => \$noprivs,
) || pod2usage ('-verbose' => 1, '-exitval' => 1);

pod2usage ('-verbose' => 1, '-exitval' => 1)
	unless ($binary && $configdir && $configfile && $pidfile && $rootdir);

print &makeSysVScript (
	'name'        => "idxpki-httpd",
	'description' => "HTTP server for IDX-PKI",
	'comment'     => "This server answers requests initiated both by ".
		"humans and third-party applications, performing tasks such as ".
		"using, administering or configuring IDX-PKI",
    'init'    => qq'if [ -f "$crlpath/crl-bundle.pem" ]; then crldef="-D WITH_CRL"; else crldef=""; fi',
	'start'       => <<"STARTCMD",
$binary -f $configfile -d $rootdir \$crldef
STARTCMD
	'details'     => "on ports HTTPS $porthttps & ".
		"HTTP $porthttp",
	# Privileges are set in the config file.
	'user'        => $noprivs ? undef : "root",
	'env'         => {'PHPRC' => $configdir},
	'pidfile'     => $pidfile,
	'asyncstart'  => 1,
);
