#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: mkphp-ini.pl,v 1.7 2003/07/25 09:34:44 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/etc/mkphp-ini.pl,v $

=pod "

=head1 NAME

mkphp-ini.pl - Generates a PHP configuration file

=head1 SYNOPSIS

B<mkphp-ini.pl> [B<--logsdir> I<directory>]

=head1 DESCRIPTION

Generates a PHP configuration file and writes it on the standard output.

If B<--logsdir> is not defined by configuration variables or the file
F<config.status> does not exist, it must be provided on the command-line.

=cut "

use strict;

use Getopt::Long;
use Pod::Usage;
use FindBin;

# Include installation tools.
use vars qw ($topdir);
$topdir = "$FindBin::RealBin/../../";
push (@INC, "$topdir/tools");
require InstallLib;
import  InstallLib ();

# Program global variables.
use vars qw (%CONFIG $logsdir $vardir);
%CONFIG = parseConfigStatus ($topdir);
$logsdir = $CONFIG{'CONFIG_LOGSDIR'};
$vardir = $CONFIG{'CONFIG_VARDIR'};

GetOptions (
	'logsdir=s'       => \$logsdir,
) || pod2usage ('-verbose' => 1, '-exitval' => 1);
pod2usage ('-verbose' => 1, '-exitval' => 1) unless ($logsdir);

my $config = << "CONFIGFILE";
; Configuration file for PHP.

; This code was developped by IDEALX (http://IDEALX.org/).
; 
; Copyright  2000-2003 IDEALX
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License version 2 as
; published by the Free Software Foundation.
; 
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
; General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
; USA.

[PHP]

; Language Options

engine			=	On
short_open_tag	=	On
asp_tags		=	Off
precision		=	14
y2k_compliance	=	Off	; whether to be year 2000 compliant (will cause problems with non y2k compliant browsers)
output_buffering	= Off


implicit_flush		= Off	
allow_call_time_pass_reference	= Off

; Safe Mode
safe_mode		=	On
safe_mode_exec_dir	=       /no/execs/allowed
safe_mode_allowed_env_vars = PHP_	
disable_functions	=

; Colors for Syntax Highlighting mode

highlight.string	=	#DD0000
highlight.comment	=	#FF8000
highlight.keyword	=	#007700
highlight.bg		=	#FFFFFF
highlight.default	=	#0000BB
highlight.html		=	#000000

; Misc
expose_php	=	On

;;;;;;;;;;;;;;;;;;;
; Resource Limits ;
;;;;;;;;;;;;;;;;;;;

max_execution_time = 30
memory_limit = 8M

error_reporting	=	E_ALL & ~E_NOTICE
display_errors	=	On
display_startup_errors = Off
log_errors		=	On
track_errors	=	On
error_log	=	$logsdir/php.log
warn_plus_overloading	=	Off

;;;;;;;;;;;;;;;;;
; Data Handling ;
;;;;;;;;;;;;;;;;;
; Note - track_vars is ALWAYS enabled as of PHP 4.0.3
variables_order		=	"EGPCS"
register_globals	=	On
register_argc_argv	=	On
post_max_size		=	8M
gpc_order			=	"GPC"

; Magic quotes
magic_quotes_gpc	=	Off	
magic_quotes_runtime=	Off
magic_quotes_sybase	=	Off

auto_prepend_file	=
auto_append_file	=

default_mimetype = "text/html"
;default_charset = "iso-8859-1"

;;;;;;;;;;;;;;;;;;;;;;;;;
; Paths and Directories ;
;;;;;;;;;;;;;;;;;;;;;;;;;
include_path		=
doc_root		=
user_dir		=
;extension_dir	=	/usr/lib/php
enable_dl		= Off

;;;;;;;;;;;;;;;;
; File Uploads ;
;;;;;;;;;;;;;;;;
file_uploads	= Off

;;;;;;;;;;;;;;;;;;
; Fopen wrappers ;
;;;;;;;;;;;;;;;;;;
allow_url_fopen = On                ; Wheter to allow treating URLs like http:... or ftp:... like files

[syslog]
define_syslog_variables	= Off

; extension=pgsql.so
CONFIGFILE


print $config;
