#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: CommonVars.pm,v 1.13 2003/03/03 14:29:03 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/CommonVars.pm,v $

=pod "

=head1 NAME

IDX::PKI::CommonVars.pm - Variables for the use of various perl scripts

=cut "

package IDX::PKI::CommonVars;

use IDX::PKI::ValidateInput; # for %wf_functions

use Exporter;
use vars qw (@ISA @EXPORT @EXPORT_OK);
@ISA = qw (Exporter);
@EXPORT = qw (
	%ldap_names @extensions
	$default_validate $default_validate_search %howto_validate
);
@EXPORT_OK = @EXPORT;

=pod "

=head1 VARIABLES

=over 4

=item I<%ldap_names>

LDAP names for DN elements: standard name => LDAP name.

=cut "

use vars qw (%ldap_names);
%ldap_names = (
	"commonName"             => "CN",
	"uniqueIdentifier"       => "UID",
	"title"                  => "T",
	"description"            => "D",
	"organizationalUnitName" => "OU",
	"organizationName"       => "O",
	"localityName"           => "L",
	"stateOrProvinceName"    => "ST",
	"countryName"            => "C",
);


=pod "

=item I<@extensions>

Certificate extensions list.

=cut "

use vars qw (@extensions);
@extensions = (
	# Key Usage Extension
	"digitalSignature",
	"nonRepudiation",
	"keyEncipherment",
	"dataEncipherment",
	"keyAgreement",
	"keyCertSign",
	"cRLSign",
	"encipherOnly",
	"decipherOnly",
	# Extended Key Usage Extension
	"emailProtection",
	"clientAuth",
	"serverAuth",
	"codeSigning",
	"timeStamping",
	"OCSPSigning",
	# Netscape Extensions
	"NSclient",
	"NSserver",
	"NSemail",
	"NSobjsign",
	"NSreserved",
	"NSsslCA",
	"NSemailCA",
	"NSobjCA",
);


=pod "

=item I<$default_validate>

Default function to use to validate a variable.

=cut "

use vars qw ($default_validate);
$default_validate = $wf_functions{'text'};


=pod "

=item I<$default_validate_search>

Default function to use to validate a variable for a LDAP search.

=cut "

use vars qw ($default_validate_search);
$default_validate_search = $wf_functions{'LDAP_wildcard'};


=pod "

=item I<%howto_validate>

Function to use to validate the considered variable (input form).

=cut "

use vars qw (%howto_validate);
%howto_validate = (
	"emailAddress"            => $wf_functions{'email'},
	"countryName"             => $wf_functions{'country'},
	"stateOrProvinceName"     => $wf_functions{'name'},
	"localityName"            => $wf_functions{'name'},
	"commonName"              => $wf_functions{'name'},
);


=pod "

=back

=cut "

######################## TEST CASE ###################################

eval join ('', <main::DATA>) || die $@ unless (caller());

1;

__END__

package IDX::PKI::CommonVars;
use strict;
use IDX::Test;

# No tests in this package: It only defines variables.
number_of_tests (0);

1;
