#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::Crypto::ASN1> - ASN.1 Functions

=head1 DESCRIPTION

This module contains some functions for generating ASN.1 extensions.

=cut "

package IDX::PKI::Crypto::ASN1;

$IDX::PKI::Crypto::ASN1::VERSION = '0.01';

use strict;
use Convert::ASN1;

use Exporter;
use vars qw(@ISA @EXPORT @EXPORT_OK);
@ISA = qw( Exporter );

@EXPORT = qw(email dns uri getBinary sequence);

@EXPORT_OK = @EXPORT;


=pod "

email($value)

Returns the DER encoding of the rfc822name value for Subject
Alternative Name Extension.

=cut "

sub email {
    my (@mails) = @_;
	my @der;
	foreach my $mail (@mails) {
		my $asn = Convert::ASN1->new();
		$asn->prepare (q< email [CONTEXT 1] IA5String >);
		my $upn = $asn->encode('email' => $mail);
		push @der, sprintf ("DER:%s", unpack('H*', $upn)) if($upn);
	}
	return @der;
}

=pod "

dns($value)

Returns the DER encoding of the dnsname value for Subject
Alternative Name Extension.

=cut "

sub dns {
    my (@dns) = @_;
	my @der;
	foreach my $dns (@dns) {
		my $asn = Convert::ASN1->new();
		$asn->prepare (q< dns [CONTEXT 2] IA5String >);
		my $upn = $asn->encode('dns' => $dns);
		push @der, sprintf ("DER:%s", unpack('H*', $upn)) if($upn);
	}
	return @der;
}

=pod "

uri($value)

Returns the DER encoding of the dnsname value for Subject
Alternative Name Extension.

=cut "

sub uri {
    my (@uris) = @_;
	my @der;
	foreach my $uri (@uris) {
		my $asn = Convert::ASN1->new();
		$asn->prepare (q< uri [CONTEXT 6] IA5String >);
		my $upn = $asn->encode('uri' => $uri);
		push @der, sprintf ("DER:%s", unpack('H*', $upn)) if($upn);
	}
	return @der;
}

=pod "

getBinary($value, ...)

=cut "

sub getBinary {
  my (@values) = @_;
  my @r;
  foreach my $v (@values) {
	  if($v =~ m/^DER:(.*)/) {
		  push @r, pack('H*', $1)
	  } else {
		  push @r, $v;
	  }
  }
  return (wantarray) ? @r : join '', @r ;
}

=pod "

sequence($value)

Returns the DER encoding of a sequence.

=cut "

sub sequence {
	my (@values) = @_;
	my $value = getBinary(@values);
	my $pdu = Convert::ASN1::asn_encode_tag(0x30).
	  Convert::ASN1::asn_encode_length(length($value)).$value;
	return(sprintf("DER:%s", unpack('H*', $pdu)));
}

eval (join '', <main::DATA>) || die $@ unless (caller ());

1;

__END__

####################  TEST SUITE  ####################

use strict;

use IDX::Test;

my $tests = 1;

number_of_tests ($tests);

my $tmp;


test "sequence" => sub {
	$tmp = sequence(email("nobody\@home.com"),
					dns("www.home.com"),
					uri("http://www.home.com"));
	TEST(length($tmp) > 8);
	TEST($tmp =~ m/DER:/);
};

