#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: Hash.pm,v 1.2 2003/01/07 17:06:38 laura Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Crypto/Hash.pm,v $

package IDX::PKI::Crypto::Hash;

$IDX::PKI::Crypto::Hash::VERSION = '0.0.1';

use strict;
use IDX::PKI::Crypto::Tools qw(file2string test_args run_openssl run_data_pipe_openssl);

my %vars = ( algo => "sha1");	# Hash algorithm

sub new {
	my $that = shift;
	my $args = { @_ };
	my $class = ref($that) || $that;
	my $self = {
				%vars,
			   };

	bless $self, $class;

	test_args( $args, "-algo");

	if (defined $args->{-algo}) {

		my %algo = (
				 "md5"        => 1,
				 "md4"        => 1,
				 "md2"        => 1,
				 "sha1"       => 1,
				 "sha"        => 1,
				 "mdc2"       => 1,
				 "ripemd160"  => 1,
				 "dss1"       => 1,
				);

		throw IDX::PKI::Error::Crypto("Invalid Algorithm".$args->{-algo})
		  unless ($algo{$args->{-algo}});

		$self->{algo} = $args->{-algo};
	}
	return $self;
}

sub hash {
	my ($self, @args) = @_;
	
	my @results;

	foreach my $data (@args) {
		my ($hash, $error) = run_data_pipe_openssl
		  ($data, "dgst", "-".$self->{algo}, "-hex");

		throw IDX::PKI::Error::Crypto ("Couldn't hash", '-details' => $error)
		  unless ($error eq "");
		chomp($hash);
		push @results, $hash;
	}

	if((scalar @args) == 1) {
		return $results[0];
	} else {
		return @results;
	}
}

sub hash_file {
	my ($self, @args) = @_;
	
	my @results;

	my ($hash, $error) = run_openssl
	  ("dgst", "-".$self->{algo}, "-hex", @args);
	
	throw IDX::PKI::Error::Crypto ("Couldn't hash", '-details' => $error)
	  unless ($error eq "");

	(@results) = ($hash =~ m/^.*= ([a-f0-9]+)$/mg);

	if((scalar @args) == 1) {
		return $results[0];
	} else {
		return @results;
	}
}

eval(join '',<main::DATA>) || die $@ unless caller();

1;

__END__

  ###########################################################"

package IDX::PKI::Crypto::Hash;
use strict;

use IDX::Test;
use IDX::PKI::Debug(1);
use IDX::PKI::Crypto::Tools;
use MIME::Base64 qw(decode_base64);

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests (5);

my $hash;
my $same_hash;
my $other_hash;

my $string1 = "This is a relatively small string";
my $string2 = << "EXAMPLE";
MIIDRTCCAu+gAwIBAgIBADANBgkqhkiG9w0BAQQFADCBkzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsT
E0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMGA1UEAxMMVGVz
dCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbTAeFw0wMTA3MDQw
NzA4NTJaFw0wMjA3MDQwNzA4NTJaMIGJMQswCQYDVQQGEwJGUjEOMAwGA1UEBxMF
UGFyaXMxFTATBgNVBAoTDFRlc3QgQ29tcGFueTEcMBoGA1UECxMTQ29tcHV0ZXIg
UmVzc291cmNlczETMBEGA1UEAxMKVGVzdENsaWVudDEgMB4GCSqGSIb3DQEJARYR
Y2xpZW50QGlkZWFseC5jb20wXDANBgkqhkiG9w0BAQEFAANLADBIAkEArDINNjS9
QcEr77DbQ7+M+s1z5l8gLqvHiaSk6mzWqHU+kgPfyFDPqXwBGjXp7f1ZutX6eFnj
kjeH9yoV8TH1RQIDAQABo4IBNDCCATAwCQYDVR0TBAIwADAdBgNVHQ4EFgQUQECZ
bsO/lDCeBXALb6WX4r2iN4owgcAGA1UdIwSBuDCBtYAUBO2qADyE7qqeOmqyRSCi
ztd3DcShgZmkgZYwgZMxCzAJBgNVBAYTAkZSMQ4wDAYDVQQHEwVQYXJpczEVMBMG
A1UEChMMVGVzdCBDb21wYW55MRwwGgYDVQQLExNDb21wdXRlciBSZXNzb3VyY2Vz
MQwwCgYDVQQLEwNQS0kxFTATBgNVBAMTDFRlc3QgUm9vdCBDQTEaMBgGCSqGSIb3
DQEJARYLY2FAdGVzdC5jb22CAQAwHAYDVR0RBBUwE4ERY2xpZW50QGlkZWFseC5j
b20wFgYDVR0SBA8wDYELY2FAdGVzdC5jb20wCwYDVR0PBAQDAgXgMA0GCSqGSIb3
DQEBBAUAA0EAiYn3CHxKPHn3touS9gf4fZ8ZJ5OLYpB22jSCb8cKpOAuuoYypoII
50CALzW+lGrJD/fKk6t3jg7iDyK0xZ6lmg==
EXAMPLE

my $binary = MIME::Base64::decode_base64($string2);

my $result1;
my $result2;

my $file1 = new IDX::Test::Filename();
my $file2 = new IDX::Test::Filename();

test "Constructor" => sub {
	$hash = new IDX::PKI::Crypto::Hash();
	$same_hash = new IDX::PKI::Crypto::Hash(-algo => "sha1");
	$other_hash = new IDX::PKI::Crypto::Hash(-algo => "md5");
	TEST(defined($hash) and defined($same_hash) and defined($other_hash));
};

test "Simple Hashing" => sub {
	$result1 = $hash->hash($string1);
	$result2 = $same_hash->hash($string1);
	TEST($result1 eq $result2);
};

test "Hash Length" => sub {
	$result1 = $hash->hash($string1);
	$result2 = $other_hash->hash($string1);
	TEST(length($result1) == 40);
	TEST(length($result2) == 32);
};

test "Binary Hashing" => sub {
	$result1 = $hash->hash($binary);
	$result2 = $same_hash->hash($binary);
	TEST($result1 eq $result2);
};

test "File Hashing" => sub {
	string2file($binary, $file1);
	string2file($binary, $file2);
	$result1 = $hash->hash($binary);
	my (@results) = $hash->hash_file($file1, $file2);

	TEST($results[0] eq $results[1]);
	TEST($results[0] eq $result1);
};

1;
