#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: Debug.pm,v 1.6 2003/07/23 13:20:44 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Debug.pm,v $

=pod "

=head1 NAME

IDX::PKI::Debug - Printing debug messages

=head1 SYNOPSIS

 use IDX::PKI::Debug (1);

 DEBUG();

 DEBUG("Debug message.");

 DEBUG($object);  # Correctly dumps references

The system environment variable I<PKI_DEBUG> might also be set to C<0> or
C<1>.  If this is the case it will override any file local settings.

=cut "

package IDX::PKI::Debug;

use strict;

# Only here: we need to stuff things into the caller's namespace
# and we don't do it the usual (Exporter.pm) way. See "import".
no strict "refs";

=pod "

=head1 METHODS

=over 4

=item I<import()>

Imports the C<DEBUG> function into caller's namespace (see below).
Automatically called when one says C<< use IDX::PKI::Debug >>.

=cut "

sub import {
	my ($pkg,$mustdebug)=@_;

	# The value of $mustdebug as specified by "use IDX::PKI::Debug (mustdebug)"
	# is overridden if the system environment variable PKI_DEBUG
	# is set to 0 or 1.
	if (defined($ENV{'PKI_DEBUG'})) {
		if ($ENV{'PKI_DEBUG'} eq '1') {
			$mustdebug = 1;
		}
		if ($ENV{'PKI_DEBUG'} eq '0') {
			$mustdebug = 0;
		}
	}

	my $callpkg=caller(0);
	*{"$callpkg\::DEBUG"}= ($mustdebug ? \&_DEBUG : (sub {}));
}


=pod "

=item I<_DEBUG($message)>

If the module was imported using C<use IDX::PKI::Debug (0);>, has no
effect.  Otherwise, prints I<$message> or (if I<$message> is a reference)
the entire hierarchy of its contents.

Without argument, just prints a friendly message saying all is well and
guess where I am calling you from.

=cut "

sub _DEBUG {
	my (@msg)=@_;
	my ($pkg,$filename,$line)=caller(0);

	foreach my $msg (@msg) {
		if (!defined $msg) {
			$msg="I am at $filename line $line\n";
		} elsif (ref($msg)) {
			require Data::Dumper;
			$msg="At $filename line $line:\n".Data::Dumper::Dumper($msg);
		} elsif ($msg !~ m/\n$/s) {
			$msg .= " at $filename line $line\n";
		}
		warn($msg);
	}
}

=pod "

=item I<IDX::PKI::Debug::isActive()>

Returns true iff debug is activated inside the caller's package.

=cut "

#pod ends "

sub isActive {
	my ($callpkg,undef,undef)=caller(0);

	return (\&{"$callpkg\::DEBUG"} eq \&_DEBUG);
}

=pod "

=back

=cut "

1;

eval join('',<main::DATA>) || die $@ unless caller();

__END__

###################### TEST CASE #######################"

use IDX::Test;

number_of_tests(2);

test "isActive" => sub {
	package Zoinx;
	import IDX::PKI::Debug (1);
	IDX::Test::TEST(IDX::PKI::Debug::isActive());
};
test "is (not) Active" => sub {
	package Blob;
	import IDX::PKI::Debug (0);
	IDX::Test::TEST(! IDX::PKI::Debug::isActive());
};
