#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: Group.pm,v 1.1 2003/05/20 14:37:12 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Group.pm,v $

=pod "

=head1 NAME

B<IDX::PKI::Goup> - Tools for handling user groups

=head1 SYNOPSIS

 use IDX::PKI::Group;

 $group = new IDX::PKI::Group("PRIV_USE", $acl);

=head1 DESCRIPTION

=cut "

package IDX::PKI::Group;
use strict;

use IDX::PKI::Authentication;
use IDX::PKI::Error;
use IDX::PKI::Config;
use IDX::PKI::DN;

sub new {
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") unless
		(scalar(@_) eq 3);

	my ($class, $operation, $section) = @_;
	my $config = the IDX::PKI::Config ();

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
		(my $name = $IDX::PKI::Authentication::privilegeNames{$operation});

	my @users = $config->getPath($section,"can-$name");

	return bless({ users => \@users }, $class);
}

sub list {
	my ($self) = @_;
	return @{$self->{users}};
}

sub isMember {
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") unless
		(scalar(@_) eq 2);
	my ($self, $user) = @_;
	my @dn = parse IDX::PKI::DN($user);
	foreach my $authorized (@{$self->{users}}) {
		return 1 if (IDX::PKI::DN::compare($authorized, \@dn));
	};
	return 0;
}

eval(join '',<main::DATA>) || die $@ unless caller;

1;

__END__

#################### TEST SUITE #########################

use IDX::Test;
use IDX::PKI::Authentication;

my $config = set IDX::PKI::Config (<< "CONFIG");
[bogusmodule]
can-change-privs=CN=John R. Hacker, DC=example, DC=com
can-change-privs=CN=God Himself, O=too many symbolic links
can-mess-up-with=CN=John Doe, DC=example, DC=com
can-configure=CN=John Doe, DC=example, DC=com

[zonkmodule]
can-change-privs=CN=John Doe/DC=example/DC=com
can-change-privs=CN=John Doeuf/DC=example/DC=com
CONFIG

number_of_tests(4);

my $group;

test "Constructor" => sub {
	TEST($group = new IDX::PKI::Group("PRIV_CHPRIV", "bogusmodule"));
};

test "list" => sub {
	TEST(scalar($group->list()) == 2);
};

test "isMember" => sub {
	TEST($group->isMember("CN=John R. Hacker, DC=example, DC=com"));
};

test "isMember" => sub {
	TEST(!$group->isMember("CN=John Doe, DC=example, DC=com"));
};

