#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: Mailbox.pm,v 1.13 2003/01/07 17:06:39 laura Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Log/Mailbox.pm,v $

package IDX::PKI::Log::Mailbox;
use strict;
use vars qw(@ISA);

use Fcntl qw(:DEFAULT :flock);
use POSIX;
use IDX::PKI::Debug;
use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Log;

@ISA = qw(IDX::PKI::Log);

=pod "

=head1 METHODS

=over 4

=item I<new()>

Object constructor.  Creates its new object with a
B<IDX::PKI::Transport::Syslog> syslog type (that is, the logging will be
done via B<IDX::PKI::Transport::Syslog>).

=cut "

sub new {
	my ($this, $section) = @_;

	my $class = ref($this) || $this;
	my $config = the IDX::PKI::Config;
	my $file = $config->getPath($section, 'file');

	throw IDX::PKI::Error::Config("Configuration incomplete")
		unless (defined $file);

	my $self = {'file' => $file};
	bless ($self, $class);

	return $self;
}

=pod "

=item I<log($message)>

Logs a message.

=cut "

sub doLog {
	my ($self, $type, %args) = @_;
	# $type is mandatory
	throw IDX::PKI::Error::Internal ("INCORRECT_ARG") unless (defined $type);
	my $file = $self->{'file'};
	
	my $text = "";
	my $subject = $args{-subject};
	my $message = "";
	
	foreach my $detail (@{$args{-details}}) {
		$message .= "Detail: ".IDX::PKI::Log::encode($detail)."\n";
	}
	
	foreach my $attach (@{$args{-attachment}}) {
		my $data;
		use Data::Dumper;
		my @datas = ((ref($attach) eq "ARRAY") ? @{$attach} : ($attach));
		$data = shift @datas if((scalar @datas) % 2);
		my %datas = ( @datas );
		$data = $datas{'-data'} unless defined $data;
		
		$message .= "<<\nAttachement: ".
		  ((defined $datas{'-name'}) ? $datas{'-name'} : "")."\n".
		  (UNIVERSAL::can($data, 'serialize') ? "Type: ".ref($data)."\n" : "").
		  IDX::PKI::Log::encode($data)."\n>>\n";
	}

	my $logfile;
	sysopen($logfile,$file,O_WRONLY|O_CREAT) ||
	  throw IDX::PKI::Error::IO(-IOfile => $file);

	return throw IDX::PKI::Error::IO("Locking error", -IOfile=>$file)
	  unless (flock($logfile,LOCK_EX|LOCK_NB));

	seek $logfile, 0, SEEK_END;

	use POSIX qw(strftime);
	my $time = (strftime "%a %b %e %H:%M:%S %Y", localtime);
	print $logfile 'From idxpki@localhost
Date: '.$time.'
Subject: Log with type '.$type.($subject ? ': "'.$subject.'"' : '').'

'.$message.'
';
	close ($logfile);
}

eval join("", <main::DATA>) || die $@ unless caller();
1;

__END__

	########## Test Suite ##########
package Bogus::Log;
sub new {	shift; return bless({@_},'Bogus::Log'); }
sub serialize { return "I'm a bogus object\n"; }

use IDX::Test;
use IDX::PKI::Config;
use IDX::PKI::Error;
use IO::File;

my $file = new IDX::Test::Filename();
my $config = set IDX::PKI::Config (<< "EOF");
[log]
type=Mailbox
file=$file
EOF

my $log;

number_of_tests(6);

test "New logging object" => sub {
	$log = new IDX::PKI::Log::Mailbox ("log");
};

test "Logging (effectiveness of logging not checked)" => sub {
	$log->alert (-subject => "Problem there");
};

test "Multiple Logging (effectiveness of logging not checked)" => sub {
	$log->debug (-details => ["Problem there", "More details"]);
};

test "Complex Logging (effectiveness of logging not checked)" => sub {
	my $Bloh = new Bogus::Log();

	$log->debug(-details => ["Some details", "More details"],
				-attachment => ["Blah",
								["BlaBla"],
								["Blih", -name => "Blih2"],
								[-data => $Bloh, -name => "Bloh"]]);
};

test "Attempting (effectiveness of logging not checked)" => sub {
	my $Bloh = new Bogus::Log();

	$log->attempt(["Working", -details => [ "Some details" ] ],
				 sub {
					 my $data = "Result";
					 my $bogus = new Bogus::Log();
					 return { -attachment => [ $data, $bogus ] };
				 });
};

test "Attempting (effectiveness of logging not checked)" => sub {
	my $Bloh = new Bogus::Log();

	try {
	$log->attempt([-subject => "Working", -details => [ "Some details" ]],
				 sub {
					 my $data = "Result";
					 my $bogus = new Bogus::Log();
					 throw IDX::PKI::Error::Internal("FAILED");
					 return { -attachment => [ $data, $bogus ] };
				 });
    } catch IDX::PKI::Error::Internal with {
		my ($E) = @_;
		TEST($E);
	};
};


#use IDX::PKI::Crypto::Tools;
#print IDX::PKI::Crypto::Tools::file2string($file);

1;
