#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: Syslog.pm,v 1.19 2003/07/23 13:20:45 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Log/Syslog.pm,v $

package IDX::PKI::Log::Syslog;
use strict;
use vars qw(@ISA);

use IDX::PKI::Debug;
use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Log;
use Sys::Syslog;

@ISA=qw(IDX::PKI::Log);

=pod "

=head1 METHODS

=over 4

=item I<new()>

Object constructor.  Creates its new object with a
B<IDX::PKI::Transport::Syslog> type (that is, the logging will be
done via syslog).

=cut "

sub new {
	my ($this, $section) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS") if (ref $section);

	$section ||= "log";
	my $config = the IDX::PKI::Config ();

	my $class = ref($this) || $this;

	my $self = bless {
		"facility" => (scalar ($config->val ($section, 'facility'))
					   || "daemon"),
		"logopt" => (scalar ($config->val ($section, 'logopt')) || ""),
	}, $class;

	Sys::Syslog::setlogsock('unix') ||
			throw IDX::PKI::Error::Network("Cannot tune syslog");
	openlog("IDX-PKI", $self->{'logopt'}, $self->{'facility'});
	return $self;
}

=pod "

=item I<doLog($type, %args)>

Logs an event.

=cut "

sub doLog {
	my ($self, $type, %args) = @_;
	# $type is mandatory
	throw IDX::PKI::Error::Internal ("INCORRECT_ARG") unless (defined $type);

	my %levels = (debug  => 'debug',
				   alert  => 'alert',
				   action => 'info' );

	my $priority = $levels{$type} || 'info';
	my $subject = $args{-subject};
	my $message = "";
	$message .= "Subject: $subject\n" if($subject);

	$args{-details} = [$args{-details}] if (ref ($args{-details}) ne "ARRAY");
	foreach my $detail (@{$args{-details}}) {
		$message .= "Detail: ".IDX::PKI::Log::encode($detail)."\n";
	}

	$args{-attachment} = [$args{-attachment}]
		if (ref ($args{-attachment}) ne "ARRAY");
	foreach my $attach (@{$args{-attachment}}) {
		my $data;
		my @datas = ((ref($attach) eq "ARRAY") ? @{$attach} : ($attach));
		$data = shift @datas if ((scalar @datas) % 2);
		my %datas = (@datas);
		$data = $datas{'-data'} unless (defined $data);

		$message .= "Attachement: ".
		  ((defined $datas{'-name'}) ? $datas{'-name'} : "")."\n".
		  (UNIVERSAL::can($data, 'serialize') ? "Type: ".ref($data)."\n" : "").
		  IDX::PKI::Log::encode($data)."\n";
	}

	$message = "<empty log message>" unless ($message);
	# Sys::Syslog::syslog is quite unhappy with empty messages.

	foreach my $line (split '\n', $message) {
		next if ($line !~ m/\S/);
		syslog($priority, $line);
	}

	DEBUG($message);
	1;
}

=pod "

=item I<end()>

Ends with the logging.

=cut "

sub end {
	closelog();
	1;
}

=pod "

=back

=cut "

eval join("", <main::DATA>) || die $@ unless caller();

1;

__END__


	########## Test Suite ##########
package Bogus::Log;
sub new {	shift; return bless({@_},'Bogus::Log'); }
sub serialize { return "I'm a bogus object\n"; }

use IDX::Test;
use IDX::PKI::Config;
use IDX::PKI::Error;
use IO::File;

my $config = set IDX::PKI::Config (<< "EOF");
[syslog]
facility=user
logopt=pid
EOF

my $log;

number_of_tests(5);

test "New logging object" => sub {
	$log = new IDX::PKI::Log::Syslog ("syslog");
};

test "Logging (effectiveness of logging not checked)" => sub {
	$log->alert(-subject => "Problem there");
};

test "Multiple Logging (effectiveness of logging not checked)" => sub {
	$log->debug(-details => ["Problem there", "More details"]);
};

test "Complex Logging (effectiveness of logging not checked)" => sub {
	my $Bloh = new Bogus::Log();

	$log->debug(-details => ["Some details", "More details"],
				-attachment => ["Blah",
								["BlaBla"],
								["Blih", -name => "Blih2"],
								[-data => $Bloh, -name => "Bloh"]]);
};

test "Closing logs" => sub {
	$log->end();
};
