#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::Publish> - IDX-PKI publication module

=head1 SYNOPSIS

 use IDX::PKI::Publish;
 my $publish = new IDX::PKI::Publish();

=head1 DESCRIPTION

This module provides methods to help the publication workflow.
They are mainly used by the script F<publish.pl>.

=cut "

package IDX::PKI::Publish;

use strict;
use Carp;

$IDX::PKI::Publish::VERSION = 'O.O.1';

use IDX::PKI::Error;
use IDX::PKI::Config;
use IDX::PKI::LDAP;

=pod "

=head1 METHODS

=over 4

=item I<new()>

Object constructor.
It just retrieves the publish section configuration.

=cut "

sub new {
	my ($that) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless(defined $that);
	my $class = ref($that) || $that;
	my $config = the IDX::PKI::Config();
	my $self = {};

	$self->{'config'} = $config;
	$self->{'ee_url'} = $config->getval("publish", "ee_url");
	$self->{'transport'} = $config->getval("publish", "publish_transport_in");
	bless($self, $class);
	return $self;
}

=pod "

=item I<getConfig()>

Returns the B<IDX::PKI::Config> object.

=cut "

sub getConfig {
	my ($self) = @_;

	return $self->{'config'};
}

=pod "

=item I<getEEURL()>

Returns the EE URL set in the config.

=cut "

sub getEEURL {
	my ($self) = @_;

	return $self->{'ee_url'};
}

=pod "

=item I<getTransport()>

Returns the type of transport set in the config.

=cut "

sub getTransport {
	my ($self) = @_;

	return $self->{'transport'};
}


require IDX::Test::Suite unless (caller());
1;

__END__

####################### TEST SUITE ###############################

package IDX::PKI::Publish;

use strict;

use IDX::PKI::Config;
use IDX::Test;
use IDX::Test::LDAPServer;
use IDX::PKI::Error;

my $tests = 3;


number_of_tests($tests);

# Overrite the "real" config with ours.
my $config = set IDX::PKI::Config (<< "EOCONF");

[publish]
# Parameters for generating URLs in emails
ee_url=https://pkidevel.ird.idealx.com:29065/
# Type: TRANSPORT-IN
transport=publish
EOCONF

my $publish;

test "Object constructor" => sub {
	$publish = new IDX::PKI::Publish();
};

test "Get EE URL" => sub {
	TEST ($publish->getEEURL() eq 'https://pkidevel.ird.idealx.com:29065/');
};

test "Get Transport" => sub {
	TEST ($publish->getTransport() eq 'publish');
};


1;
