#!/usr/bin/perl -w

# $Id: Local.pm,v 1.7 2003/01/07 17:06:40 laura Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Transport/Local.pm,v $

################################
## IDX::PKI::Transport::Local ##
################################
package IDX::PKI::Transport::Local;

use strict;
use vars qw(@ISA);
use IDX::PKI::Transport;

@ISA=qw(IDX::PKI::Transport);

########################################
## IDX::PKI::Transport::Local::Sender ##
########################################
package IDX::PKI::Transport::Local::Sender;
use strict;
use vars qw(@ISA);
use IDX::PKI::Config;
use IDX::PKI::Error;

@ISA=qw(IDX::PKI::Transport::Sender);

sub new {
	my ($class, $section) = @_;
	my $config = the IDX::PKI::Config ();
	my $directory = $config->getPath($section, "directory");
	my $post = $config->getPath($section, "post");
	throw IDX::PKI::Error::Internal ("Configuration is incomplete.")
		unless (defined $directory);
	my $self = { "directory" => $directory,
							 "section" => $section,
							 "post" => $post };
	return bless($self, $class);
}

sub send {
	no strict 'refs';
	my ($self,$msg)=@_;
	my $message = $self->encode_message($msg);

	my $id = IDX::PKI::Transport::write_spoolx($message, $self->{directory});

	if($self->{post})
		{
			system($self->{post}, $self->{directory}."/".$id);
		}

	return "QUEUED $id";
}

######## TEST CASE FOR IDX::PKI::Transport::Local::Sender AND  #############
########       IDX::PKI::Transport::Unspooler                   #############

eval(join '',<main::DATA>) || die $@ unless caller();

1;

__END__


package IDX::PKI::Transport::Local;
use IDX::PKI::Config;
use IDX::Test;
use IDX::PKI::Error;
use IDX::PKI::Debug (0);
use Data::Dumper;

number_of_tests(16);

my $bogon="lkjdgdf_\n.\r\n.\x00\\x\e"x500;

my ($transportout,$transportin);

my $transportdir=new IDX::Test::Directory();

my $config = set IDX::PKI::Config (<< "CONFIG");
[testtransport]
type=local
directory=$transportdir
CONFIG

test "configuration (sender)" => sub {
	TEST ($transportout = new IDX::PKI::Transport::Sender ("testtransport"));
};

{
package __Bogus::package1;
$__Bogus::package1::VERSION="0.01";

sub new { return bless({}); }
sub serialize { return $bogon; };
sub deserialize { my ($class,$blob)=@_;
	return bless({"blob" => $blob},$class); }
}

%ENV = ("PATH" => "/bin:/usr/bin");

test "sending" => sub {
	$transportout->send(new __Bogus::package1());
	TESTFAIL() unless (my @globs=glob("$transportdir/*"));
	# Untaint.
	my $file = $globs[0];
	$file =~ m/^([a-z0-9\-\/]+)$/i || die "Unexpected value $file";
	$file = $1;
	TEST(`cat $file` =~ m/BEGIN IDXPKI MESSAGE/);
};

test "Configuration (receiver)" => sub {
	$transportin = new IDX::PKI::Transport::Unspooler("testtransport");

};

my @list;
test "Listing" => sub {
	local *TOUCH;

	open(TOUCH,">".$transportin->{directory}."/AAAAAAAA.hammer");
	# U can touch this :-) without disturbing ->list().
	close(TOUCH);

	@list=$transportin->list();
	TEST(scalar(@list) == 1);
};

my @searchparams=($list[0],'__Bogus::package1');
my $obj;

test "reading" => sub {
	TEST($obj=($transportin->readByID(@searchparams)));
	TEST($obj->{blob} eq $bogon);
	TEST(ref($obj) eq "__Bogus::package1");
};

test "reading via anonymous constructor" => sub {
	$transportin->readByID(@searchparams,sub {TEST(shift eq $bogon)});
	0;
};

test "pulling the thing read" => sub {
	my $obj2=$transportin->pullByID(@searchparams);
	TEST( (defined $obj2) &&
		(Data::Dumper::Dumper($obj) eq Data::Dumper::Dumper($obj2)) );
};

test "reading the thing pulled" => sub {
	my $obj2=$transportin->readByID(@searchparams);
	TEST( (defined $obj2) && 
		(Data::Dumper::Dumper($obj) eq Data::Dumper::Dumper($obj2)) );
};

test "pulling again the thing pulled (shall not work)" => sub {
	my $result=$transportin->pullByID(@searchparams);
	TEST(!defined $result);
};

test "pulling from parallel process (shall fail also)" => sub {
	defined(my $pid=fork()) || die;

	if ($pid) {
		waitpid($pid,0);
		TEST(($? >> 8) == 0);
	} else {
		my $othertransport = new
			IDX::PKI::Transport::Unspooler("testtransport");
		POSIX::_exit((defined ($transportin->pullByID(@searchparams))) ?
			1 : 0 );
	};
};

test "reading from parallel process (shall succeed)" => sub {
	defined(my $pid=fork()) || die;

	if ($pid) {
		waitpid($pid,0);
		TEST(($? >> 8) == 0);
	} else {
		my $othertransport = new
			IDX::PKI::Transport::Unspooler("testtransport");
		POSIX::_exit( (defined ($transportin->readByID(@searchparams))) ?
			0 : 1 );
	};
};

test "expunging" => sub {
	$transportin->expunge();
	my @l=$transportin->list();
	TEST(0==scalar(@l));
};

test "reading after expunge (shall fail)" => sub {
	TEST(!defined ($transportin->readByID(@searchparams)));
};

my $msgid;
test "send another one" => sub {
	$transportout->send(new __Bogus::package1());
	TESTFAIL() unless (my @globs=glob("$transportdir/*"));
	my ($nonMC) = grep {! m/hammer/ } @globs;
	# Untaint.
	$nonMC =~ m/^([a-z0-9\-\/]+)$/i || die "Unexpected value $nonMC";
	$nonMC = $1;
	TEST(`cat $nonMC` =~ m/BEGIN IDXPKI MESSAGE/) ||
		warn `cat $nonMC`;

	($msgid)=$transportin->list();
};
@searchparams=($msgid,'__Bogus::package1');

test "automatic release of lock" => sub {
	do {
		my $transportinbis = new IDX::PKI::Transport::Unspooler
			("testtransport");
		TEST($transportinbis->pullByID(@searchparams));
		TEST(! $transportin->pullByID(@searchparams));
	};
	TEST($transportin->pullByID(@searchparams));
	$transportin->release();
};

test "automatic release of lock in catastrophic cases" => sub {
	  try {
		  my $transportinbis = new IDX::PKI::Transport::Unspooler
			  ("testtransport");
		  TEST($transportinbis->pullByID(@searchparams));
		  die "if you see this message, try use()ing IDX::PKI::Error";
	  } otherwise {
		  TEST($transportin->pullByID(@searchparams));
	  };
	$transportin->release();
};

1;
