#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: ca-crr-list.cgi,v 1.20 2003/07/23 13:20:48 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perlscript/CA/ca-crr-list.cgi,v $

use strict;

use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Crypto::CRR;
use IDX::PKI::Crypto::X509;
use IDX::PKI::DN;
use IDX::PKI::Transport;
use IDX::PKI::Principal;
use IDX::PKI::Authentication qw(authorize);

=pod "

=head1 NAME

CA/ca-crr-list.cgi

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews("ca-crr-list.php");

############### Configuration informations ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('CA', 'web_acl'));
my $transport = new IDX::PKI::Transport::Unspooler
  ("TRANSPORT-IN-".$config->getval("CA", "crr_transport_in"));

################ Read List of CRR #############################
my @list_of_messages;
try {
	@list_of_messages  = $transport->list();
} otherwise {
	$log->alert(TRANSPORT_MSG,
				-attachment => [ shift ]);
	$cgi->failure(TRANSPORT_MSG);
};

my @pending_crr_list = ();

foreach my $id (@list_of_messages) {
	my $crr;
	try {
		$crr = readCRR ($transport, $id);
	} otherwise {
		$log->alert(TRANSPORT_MSG, -details => [ "Message ID: $id" ]
					-attachment => [ shift ]);
	};
	next unless ($crr);
	
	my $x509 = parse IDX::PKI::Crypto::X509 ($crr->get_crr_openssl() );
	my $dn = build IDX::PKI::DN ($x509->getSubjectDN ());
	push (@pending_crr_list, ([$id, $dn->getUserID()]));
}

$cgi->success(\@pending_crr_list);
