#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Registry;
use IDX::PKI::CA;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Crypto::SignedBlob;
use IDX::PKI::Transport;
use IDX::PKI::Principal qw (pullSignedBlob);
use IDX::PKI::Crypto::Tools qw (string2file);
=pod "

=head1 NAME

B<ca-csr-accept.cgi> - Script to accept a Certificate Signing Request

=cut "

############### CGI init ####################
$cgi->setupViews ("ca-csr-accept.php");

############### Configuration informations ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('CA', 'web_acl'));

my $section   = $config->getval ("CA", "csr_transport_in");
my $transport = new IDX::PKI::Transport::Unspooler
	("TRANSPORT-IN-".$section);

################ Read pending CSR ###############################

my $id = $cgi->validate ("id", \&IDX::PKI::ValidateInput::messageID);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures() &&
							   $cgi->internalparam ("ERROR_id"));

my $signedcsr = pullSignedBlob ($transport, $id)
	|| $cgi->failure (TRANSPORT_MSG);

$signedcsr->verify ("-cadir" => $config->getval ("paths", "cacertsdir"));

my $csr = deserialize IDX::PKI::Crypto::CSR ($signedcsr->get_object ());
$log->addTracking ($csr->getFormEntry ("tracking"));

my $ca = new IDX::PKI::CA ($log);
my $x509;
try {
	$x509 = $ca->csr_accept ($csr);
} catch IDX::PKI::Error::Network with {
	my ($E) = @_;

	if ($E->{-codename} eq "LDAP_ALREADY_EXISTS") {
		# Probably a crash recovery situation
		my $dn = $csr->getDN ();
		$log->warning ("Skipped CSR for DN $dn", "-data" => $E);
	} elsif ($E->{'-text'} =~ /transport/i ||
			 $E->{'-text'} eq "MALFORMED_MESSAGE") {
		# Transport error.
		$log->alert ("TRANSPORT_MSG", "-attachment" => $E);
		$cgi->failure (TRANSPORT_MSG);
	} else {
		# LDAP error.
		$log->alert ("REPOSITORY_PUBLISH", "-attachment" => $E);
		$cgi->failure (REPOSITORY_PUBLISH);
	}
} catch IDX::PKI::Error::Config with {
	my ($E) = @_;
	$log->alert ("CONFIG_INVALID", "-attachment" => $E);
	$cgi->failure (CONFIG_INVALID);
};

my $profilename = $csr->getFormEntry ("profile");
my $fileId = $csr->getFormEntry ("fileId");

if($x509 && ($profilename eq "scep") && $fileId) {
    my $scepdir = $config->getPath("paths", "scepdir")
        || $cgi->error (CONFIG_INVALID);

    string2file($x509->serialize('DER'), "$scepdir/granted/$fileId.der");
    system("/bin/cp", "-f", "$scepdir/pending/$fileId.info",
                 "$scepdir/granted");
    system("/bin/rm", "-f",
                 "$scepdir/pending/$fileId.der",
                 "$scepdir/pending/$fileId.info",
				 "$scepdir/pending/$fileId.sent");
}

$transport->expunge();

$cgi->success ($x509->serialize ());
