#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Registry;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Crypto::SignedBlob;
use IDX::PKI::Transport;
use IDX::PKI::Profile;
use IDX::PKI::DN;
use IDX::PKI::Principal qw (readSignedBlob);

=pod "

=head1 NAME

B<ca-csr-details.cgi> - Details of a pending CSR

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews ("ca-csr-details.php");

############### Configuration informations ####################
$cgi->authorize ("PRIV_USE",
				 $config->getSectionName ("CA", ["ACL", "web_acl"]));
$log->addAuthorizer ($cgi->getPeerCertificate ());

my $section   = $config->getval ("CA","csr_transport_in");
my $transport = new IDX::PKI::Transport::Unspooler
	("TRANSPORT-IN-".$section);

################ Read pending CSR ###############################

my $id = $cgi->validate ("id", \&IDX::PKI::ValidateInput::messageID);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures() &&
							   $cgi->internalparam ("ERROR_id"));

my $signedcsr = readSignedBlob($transport, $id)
	|| $cgi->failure (TRANSPORT_MSG);

$signedcsr->verify ("-cadir" => $config->getval ("paths", "cacertsdir"));

my $csr = deserialize IDX::PKI::Crypto::CSR ($signedcsr->get_object());

my $profilename = $csr->getFormEntry ("profile");

my (@dn, %form);
use Tie::IxHash;
tie (%form, 'Tie::IxHash');

if ($profilename) {
	my $profile = new IDX::PKI::Profile ($profilename);
	# DN elements.
	my @tmpdn = build IDX::PKI::DN ($csr->getDN ())->canonical ();
	while (my ($short, $val) = splice (@tmpdn, 0, 2)) {
		my $long = IDX::PKI::DN::short_to_long ($short);
		push (@dn, $long, $val) if ($val);
	}

	# For each form element, don't show it if it is encrypted.
	my %counts;
	my %csr_form = %{$csr->{'form'}};
	foreach my $elem ($profile->getFormElements ()) {
		my $long  = $profile->elementLongName ($elem);
		my $short = $profile->elementShortName ($elem);

		$counts{$short} ++;

		next if ($profile->elementOptions ($elem, 'encrypt'));

		# Multiple occurences will appear as an array refernce.
		my $val;
		if (ref ($csr_form{$short})) {
			$val = $csr_form{$short}[$counts{$short} - 1];
		} else {
			$val = ($counts{$short} == 1) ? $csr_form{$short} : undef;
		}
		push (@{$form{$long}}, $val);
	}

	# Extensions.
	foreach my $ext ($profile->getExtensions ()) {
		$form{$ext} = 1;
	}
} else {
	@dn   = $csr->getDN();
	%form = %{$csr->{'form'}};
	delete ($form{'admin'});	# Don't show that...
}

delete ($form{'pkcsten'});	# Don't show that...

$cgi->success ([\@dn, \%form]);
