#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Registry;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Crypto::SignedBlob;
use IDX::PKI::DN;
use IDX::PKI::Transport;
use IDX::PKI::Principal qw (readSignedBlob);

=pod "

=head1 NAME

B<ca-csr-list.cgi> - List pending CSRs

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews ("ca-csr-list.php");

############### Configuration informations ####################
$cgi->authorize ("PRIV_USE",
				 $config->getSectionName ("CA", ["ACL", "web_acl"]));
$log->addAuthorizer ($cgi->getPeerCertificate ());

my $section = $config->getval ("CA", "csr_transport_in");
my $transport = new IDX::PKI::Transport::Unspooler
	("TRANSPORT-IN-".$section);

################ Read List of CSR #############################

my @list_of_messages;
try {
	@list_of_messages = $transport->list();
} otherwise {
	my ($E) = @_;
	$log->alert ("TRANSPORT_MSG",
				 "-details" => "Transport: TRANSPORT-IN-".$section,
				 "-attachment" => $E);
	$cgi->failure (TRANSPORT_MSG);
};

my @pending_csr_list = ();

foreach my $id (@list_of_messages) {
	my $signedcsr;
	try {
		$signedcsr = readSignedBlob($transport, $id);
	} otherwise {
		my ($E) = @_;
		$log->alert ("TRANSPORT_MSG",
					 "-details" => ["Transport: TRANSPORT-IN-".$section,
									"Message ID: $id"],
					 "-attachment" => $E);
	};
	next unless ($signedcsr);

	my $csr;
	try {
		$signedcsr->verify ("-cadir" =>
							$config->getval ("paths", "cacertsdir"));

		if ($signedcsr->get_type() =~ m/IDX::PKI::Crypto::CSR/) {
			$csr = deserialize
				IDX::PKI::Crypto::CSR ($signedcsr->get_object());
		} else {
			$log->alert ("TRANSPORT_MSG",
						 "-details" => ["Transport: TRANSPORT-IN-".$section,
										"Message ID: $id"],
						 "-attachment" => $signedcsr);
		}
	} otherwise {
		my ($E) = @_;
		$log->alert ('TRANSPORT_SIG',
					 "-details" => ["Transport: TRANSPORT-IN-".$section,
									"Message ID: $id"],
					 "-attachment" => [$signedcsr, $E]);
	};
	next unless ($csr);

	my $subject = $csr->getFormEntry ("subjectAltName");
	my $dn = build IDX::PKI::DN ($csr->getDN ());

	if (defined $subject) {
		push (@pending_csr_list, ([$id, $subject]));
	} else {
		push (@pending_csr_list, ([$id, $dn->getUserID()]));
	}
}

$cgi->success (\@pending_csr_list);
