#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Registry;
use IDX::PKI::Crypto::SignedBlob;
use IDX::PKI::Transport;
use IDX::PKI::Principal qw (pullSignedBlob);

=pod "

=head1 NAME

B<ca-csr-reject.cgi> - Script to reject a Certificate Signing Request

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews ("ca-csr-reject.php");

############### Configuration informations ####################
$cgi->authorize ("PRIV_USE",
				 $config->getSectionName ("CA", ["ACL", "web_acl"]));
$log->addAuthorizer ($cgi->getPeerCertificate ());

my $section = $config->getval ("CA", "csr_transport_in");
my $transport = new IDX::PKI::Transport::Unspooler
	("TRANSPORT-IN-".$section);

################ Remove pending CSR ###############################
my $id = $cgi->validate ("id", \&IDX::PKI::ValidateInput::messageID);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures() &&
							   $cgi->internalparam ("ERROR_id"));

$log->attempt ("CSR_REJECT", sub {
	my $signedcsr = pullSignedBlob ($transport, $id)
		|| $cgi->failure (TRANSPORT_NOTFOUND);

	$signedcsr->verify ("-cadir" => $config->getval ("paths", "cacertsdir"));

	my $csr = deserialize IDX::PKI::Crypto::CSR
		($signedcsr->get_object ()) || $cgi->failure (TRANSPORT_MSG);

	$log->addTracking ($csr->getFormEntry ("tracking"));

	$transport->expunge();

    my $profile = $csr->getFormEntry ("profile");
    my $fileId = $csr->getFormEntry ("fileId");
    if (($profile eq "scep") && $fileId)   {
         my $scepdir = $config->getPath("paths", "scepdir")
            || $cgi->error (CONFIG_INVALID);

         my $ret = system("/bin/mv", "-f", 
                               "$scepdir/pending/$fileId.der",
                               "$scepdir/pending/$fileId.info",
                               "$scepdir/rejected");
         $ret = system("/bin/rm","-f",
                               "$scepdir/pending/$fileId.sent");

         $cgi->error(REQUEST_INVALID)
            if ((-f "$scepdir/pending/$fileId.der") ||
                (-f "$scepdir/pending/$fileId.info"));
    }

	return {"-attachment" => $csr};
});

$cgi->success (0);
