#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: ca-scep-accept.cgi,v 1.4 2003/02/17 12:03:49 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perlscript/CA/ca-scep-accept.cgi,v $

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::CA;
use IDX::PKI::Crypto::Tools;
use IDX::PKI::Crypto::PKCS10;
use IDX::PKI::Principal;
use IDX::PKI::Authentication qw(authorize);

$cgi->setupViews ("ca-scep-accept.php");

############### Access Restriction ####################
$cgi->authorize("PRIV_USE",$config->getSectionName
				('CA', ["ACL" => 'web_acl']));

############### Configuration informations ####################
my $scepdir = $config->getPath("paths", "scepdir")
	|| $cgi->error (CONFIG_INVALID);
my $id = $cgi->validate("id" => \&IDX::PKI::ValidateInput::messageID);
my $type = $cgi->validate("type" => \&IDX::PKI::ValidateInput::choice,
						  "-extraargs" => [ "IP", "DNS", "URI"] );
my $value = $cgi->validate("value" => \&IDX::PKI::ValidateInput::uri);

$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures());
$cgi->error (INPUT_MISSING) unless ($id and $type and $value);

my ($pkcs10, $err) = run_openssl("req", "-inform", "der", "-in",
								 "$scepdir/pending/$id.der");
my @dn = parse IDX::PKI::Crypto::PKCS10 ($pkcs10)->getDN();
my $csr = new IDX::PKI::Crypto::CSR::PKCS10
	(-dn => \@dn,	 -csr => $pkcs10,
	 -form => { altNameType => $type, subjectAltName => $value });

my $ca = new IDX::PKI::CA ($log);
my $x509 = $ca->csr_accept ($csr);

if($x509) {
	string2file($x509->serialize('DER'), "$scepdir/granted/$id.der");
	system("/bin/cp", "-f", "$scepdir/pending/$id.info",
				 "$scepdir/granted");
	system("/bin/rm", "-f",
				 "$scepdir/pending/$id.der",
				 "$scepdir/pending/$id.info");
	$cgi->success($x509->serialize());
}
$cgi->error(REQUEST_INVALID);
