#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: ee-auto-receive.pl,v 1.2 2004/01/26 12:00:19 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perlscript/EE/ee-auto-receive.pl,v $

=pod "

=head1 NAME

ee-p12-receive.pl - Warns the user that his PKCS#12 is ready

=head1 USAGE

B<ee-p12-receive.pl> I<message file>

=head1 DESCRIPTION

This script warns the user that his PKCS#12 file is ready by sending an
email.  It should be run each time a new PKCS#12 message arrives on the
EE, for example as a 'post' transport rule.

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Log;
use IDX::PKI::Error;
use IDX::PKI::Transport;
use IDX::PKI::Crypto::Tools qw (file2string);
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Principal; # for sendMimeMail

use Pod::Usage;
use Carp;

my $config = the IDX::PKI::Config ();
my $log    = new IDX::PKI::Log ();

$log->attempt ({-subject => "AUTO_RECEIVE", -details => ["in $ARGV[0]"]},
	sub {
	my $file    = $ARGV[0];
	pod2usage ('-verbose' => 1, '-exitval' => 1)
	  if ((! defined $file) || ($file eq ""));
	die "Can't find message file '$file'." unless (defined $file && -f $file);
	my $csr = IDX::PKI::Transport::decode_message (undef, file2string ($file),
			'IDX::PKI::Crypto::CSR');

	my $email = $csr->getFormEntry ("emailAddress");

	if ($email) {
		my $admin_email = $config->getval ("EE", "admin_email");
		my ($id)    = ($file =~ m/([A-Za-z0-9]+)$/);
		my $url     = $config->getPath ("EE", "auto_url") . $id;
		my $message = << "MESSAGE";
[French translation at the end. / Traduction en franais  la fin.]

Hello,

Your file has been processed, in order to issue your certificate please
proceed to the following URL:
$url

          ----------------------------------------

Bonjour,

Votre dossier a t trait, pour gnrer votre certificat, suivez le
lien ci-dessous :
$url
MESSAGE

		IDX::PKI::Principal::sendMimeMail
  ( -from    => $admin_email,
	-to      => $email,
	-subject => "Enroll into PKI / Inscrivez-vous dans la PKI",
	-message => $message );
    } else {
      return {'-details' =>  [ 'No email address present, no mail sent.' ] };
    }
});
exit 0;
