#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ee-ca-incorporate.cgi> - Incorporate the CA's certificates into a browser

=head1 SYNOPSIS

=head1 DESCRIPTION

This scripts send a CA's valid file to the browser.
This file depends of the browser's type:

=over 4

=item Netscape or IE

A DER serialized PKCS7 certificate (with the CRL).

=item Other

Returns the concatenation of all CA certificates in PEM format.

=head1 PARAMETERS

One parameter is recognized:

=over 4

=item I<browser>

Browser type. If I<browser> is C<NETSCAPE> or C<IE>, the entire CA
certificate chain is returned in PKCS#7 "null signature" format.

If browser is not set or is neither of these types, the concatenation of
all CA certificates in PEM version is returned (in F<ca-bundle.pem>
style), which is most useful for "wget"ting from SSL servers within the CA
domain.

=back

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::PKCS7;
use IDX::PKI::Crypto::CRL;

$cgi->setupViews ("__NONE__");
$cgi->authorize ("__NONE__");

################ Read parameters ###############################
my $browser = $cgi->validate ("browser", qr/^([A-Za-z]+)$/i);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures() &&
							   $cgi->internalparam ("ERROR_browser"));

################ Certificate output ###############################
my ($type, $output);
if ($browser && ($browser eq "NETSCAPE" || $browser eq "IE")) {
	$type = ($browser eq "IE") ? "application/x-pkcs7-certificates" :
		"application/x-x509-ca-cert";
	my @cabundle = loaddir
		IDX::PKI::Crypto::X509 ($config->getval ("paths", "cacertsdir"));
	my @crls = loaddir
		IDX::PKI::Crypto::CRL ($config->getval ("paths", "crldir"));
	$output = build
		IDX::PKI::Crypto::PKCS7::SignedData (undef,
											 (map {('-cert', $_)} @cabundle),
											 (map {('-CRL', $_)} @crls))
		->serialize("DER");
} else {
	$type = "text/plain"; # Why bother ?
	my @cabundle = loaddir IDX::PKI::Crypto::X509
		($config->getval ("paths", "cacertsdir"));
	$output = join ("\n", map {$_->serialize ("PEM")} @cabundle);
}

$cgi->{printing_allowed} = 1;

$cgi->content_type($type);
$cgi->content_length (length ($output));
$cgi->print ($output);
exit;
