#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: ee-csr-form.cgi,v 1.24 2004/01/09 19:43:29 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perlscript/EE/ee-csr-form.cgi,v $

=pod "

=head1 NAME

B<ee-csr-form.cgi> - Returns informations to be asked to the user for a new CSR

=head1 DESCRIPTION

This CGI returns informations about the form to be filled by the user for
submitting a new certificate request.

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Profile;
use IDX::PKI::DN;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Principal qw (set_profile_parameter transmit_profile);

$cgi->setupViews (-success => "ee-csr-form.php",
				  -error => "ee-csr-form.php");
$cgi->authorize ("__NONE__");

sub set_param {
	my ($profile, $elem, $param, $value) = @_;
	if (defined $value) {
		if ($profile->elementOptions ($elem, "verify")) {
			$cgi->param ($param."a", $value);
			$cgi->param ($param."b", $value);
		} else {
			$cgi->param ($param, $value);
		}
	}
}

################ Call Parameters ###############################
my $profilename = $cgi->validate ("profile", qr/^([a-z0-9\-_]+)$/i);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures() &&
							   $cgi->internalparam ("ERROR_profile"));

my $profile;
try {
	$profile = new IDX::PKI::Profile ($profilename);
} catch IDX::PKI::Error with {
	$cgi->error (INPUT_HIDDEN);
};


my $cert = $cgi->getPeerCertificate ();
my $x509 = $cert ? parse IDX::PKI::Crypto::X509 ($cert) : undef;
my $dn = defined ($x509) ? build IDX::PKI::DN ($x509->getSubjectDN ()) : undef;

my @shown_rdn;
my %counts = ();
foreach my $rdn ($profile->getDNElements ()) {
	next if ($profile->elementOptions ($rdn, "raonly"));
	push (@shown_rdn, $rdn);
	my $short = $profile->elementShortName ($rdn);
	my $long  = $profile->elementLongName ($rdn);
	$counts{$long} ++;
	set_profile_parameter ($cgi, $profile, $rdn, $long.$counts{$long},
						   $dn->getRDNValue ($short)) if (defined $dn);
}

my @shown_form;
%counts = ();
foreach my $form ($profile->getFormElements ()) {
	next if ($profile->elementOptions ($form, "raonly"));
	push (@shown_form, $form);
	my $short = $profile->elementShortName ($form);
	my $long  = $profile->elementLongName ($form);
	$counts{$long} ++;
	# Email address is the only one we know what to do with.
	set_profile_parameter ($cgi, $profile, $form, $long.$counts{$long},
						   $x509->getEmail ())
		if ($short eq "emailAddress" && defined ($x509));
}

transmit_profile ($cgi, $profile, \@shown_rdn, \@shown_form);
$cgi->success (0);
