#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ee-csr-send.cgi> - Sends a certificate request according to a profile

=cut "

use strict;

use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Profile;
use IDX::PKI::Group;
use IDX::PKI::DN;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Crypto::PKCS10;
use IDX::PKI::Crypto::Hash;
use IDX::PKI::Crypto::PKCS7 qw(pkcs7Encrypt);
use IDX::PKI::Transport;
use IDX::PKI::ValidateInput;
use IDX::PKI::Principal qw (set_profile_parameter transmit_profile sendmail);

$cgi->setupViews ("-success" => "ee-csr-send.php",
				  "-error"   => "ee-csr-form.php",
				  "-form"    => "ee-csr-form.php",
				  "-confirm" => "ee-csr-confirm.php");
$cgi->authorize ("__NONE__");

############### Configuration informations ####################
my $administrator = $config->val ('EE', 'admin_email') ||
	$cgi->failure (CONFIG_INVALID);
my $section = $config->getval ('EE', 'csr_transport_out');
my $spam_admins = $config->val ('EE', 'spam_admins');
my $transport = new IDX::PKI::Transport::Sender ("TRANSPORT-OUT-".$section);

############### Call parameters ####################
my $profilename = $cgi->validate ("profile", qr/^([a-z0-9\-_]+)$/i);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures() &&
							   $cgi->internalparam ("ERROR_profile"));

################ Loading the profile ###############################
my $profile = new IDX::PKI::Profile ($profilename);

# Returns the profile to the PHP script.
do {
	my @shown_rdn;
	foreach my $rdn ($profile->getDNElements ()) {
		push (@shown_rdn, $rdn)
			unless ($profile->elementOptions ($rdn, "raonly"));
	}
	my @shown_form;
	foreach my $form ($profile->getFormElements ()) {
		push (@shown_form, $form)
			unless ($profile->elementOptions ($form, "raonly"));
	}
	transmit_profile ($cgi, $profile, \@shown_rdn, \@shown_form);
};

############### Checking against profile ####################
# Build elements we want to use for the request: DN elements.
my @dn = ();
my %counts = ();
foreach my $rdn ($profile-> getDNElements ()) {
	next if ($profile->elementOptions ($rdn, "raonly"));

	my $short = $profile->elementShortName ($rdn);
	my $long  = $profile->elementLongName ($rdn);
	my $req   = $profile->elementImportance ($rdn);
	$counts{$long} ++;
	my $paramname = $long.$counts{$long};

	if ($req eq "fixed") {
		# Fixed.
		push (@dn, $short, $profile->elementDefaultValue ($rdn));
	} else {
		my $type = $profile->elementType ($rdn);
		my $value;

		# Checks the value given by the user.
		my ($validatesub, @extras);
		if ($type eq "choice") {
			$validatesub = $wf_functions{'choice'};
			@extras = ($profile->elementChoices ($rdn));
		} elsif (exists $wf_functions{$type}) {
			$validatesub = $wf_functions{$type};
			@extras = ($profile->elementMaximumLength ($rdn));
		} else {
			throw IDX::PKI::Error::Internal
				("CONFIG_INVALID", "-details" => "Unknown validation type",
				 "-type" => $type, "-profile" => $profilename);
		}

		# If the user must type the value twice for confirmation.
		if ($profile->elementOptions ($rdn, "verify")) {
			$value = $cgi->validate
				(
				 $paramname,
				 [\&IDX::PKI::ValidateInput::MultiParam::confirm, "a", "b"],
				 "-extraargs" => [$validatesub, @extras],
				 ($req ne "mandatory") ? ("-default" => undef) : (),
				 );
		} else {
			$value = $cgi->validate
				(
				 $paramname,
				 $validatesub,
				 "-extraargs" => \@extras,
				 ($req ne "mandatory") ? ("-default" => undef) : (),
				 );
		}

		# Invalid value or mandatory parameter not supplied.
		if ($cgi->validateFailures() &&
			$cgi->internalparam ("ERROR_$paramname")) {
			$cgi->internalparam ("ERROR", $long);
			($cgi->internalparam ("ERROR_$paramname") eq "MISSING") ?
				$cgi->error (INPUT_MISSING) : $cgi->error (INPUT_FORMAT);
		}

		next unless (defined $value);	# Not mandatory, not filled.

		push (@dn, $short, $value);
	}
}

# Build elements we want to use for the request: Form elements.
my %forms = ("profile" => $profile->getName ());
%counts = ();
foreach my $form ($profile-> getFormElements ()) {
	next if ($profile->elementOptions ($form, "raonly"));

	my $long  = $profile->elementLongName ($form);
	my $short = $profile->elementShortName ($form);
	my $req   = $profile->elementImportance ($form);
	my $type  = $profile->elementType ($form);
	$counts{$long} ++;
	my $paramname = $long.$counts{$long};
	my $value;

	if ($req eq "fixed") {
		# Fixed.
		$value = $profile->elementDefaultValue ($form);
	} else {
		# Checks the value given by the user.
		my ($validatesub, @extras);
		if ($type eq "choice") {
			$validatesub = $wf_functions{'choice'};
			@extras = ($profile->elementChoices ($form));
		} elsif (exists $wf_functions{$type}) {
			$validatesub = $wf_functions{$type};
			@extras = ($profile->elementMaximumLength ($form));
		} else {
			throw IDX::PKI::Error::Internal
				("CONFIG_INVALID", "-details" => "Unknown validation type",
				 "-type" => $type, "-profile" => $profilename);
		}

		# If the user must type the value twice for confirmation.
		if ($profile->elementOptions ($form, "verify")) {
			$value = $cgi->validate
				(
				 $paramname,
				 [\&IDX::PKI::ValidateInput::MultiParam::confirm, "a", "b"],
				 "-extraargs" => [$validatesub, @extras],
				 ($req ne "mandatory") ? ("-default" => undef) : (),
				 );
		} else {
			$value = $cgi->validate
				(
				 $paramname,
				 $validatesub,
				 "-extraargs" => \@extras,
				 ($req ne "mandatory") ? ("-default" => undef) : (),
				 );
		}

		# Invalid value or mandatory parameter not supplied.
		if ($cgi->validateFailures() &&
			$cgi->internalparam ("ERROR_$paramname")) {
			$cgi->internalparam ("ERROR", $long);
			($cgi->internalparam ("ERROR_$paramname") eq "MISSING") ?
				$cgi->error (INPUT_MISSING) : $cgi->error (INPUT_FORMAT);
		}
	}

	next unless (defined $value);	# Not mandatory, not filled.

	# Encrypt field if necessary.
	my $encrypt = $profile->elementOptions ($form, "encrypt");
	if ($encrypt) {
		$value = pkcs7Encrypt ($value,
							   $config->getval ("certificates", $encrypt));
	}

	# Can be a single value or array reference.
	if (exists ($forms{$short}) && ref ($forms{$short}) eq "ARRAY") {
		# Already an array ref.
		push (@{$forms{$short}}, $value);
	} elsif (exists $forms{$short}) {
		# Already a text value => must become a hash.
		$forms{$short} = [$forms{$short}, $value];
	} else {
		# No need to have an array ref.
		$forms{$short} = $value;
	}
}

################ Creating the CSR ###############################
my $csr;
my $type = $profile->getType ();
my $bcsr = ($type eq "user") ? 1 : 0;

if ($bcsr) {
	my $spkac = $cgi->validate ("SPKAC", \&IDX::PKI::ValidateInput::spkac,
								"-default" => undef);
	$cgi->error (REQUEST_CREATE) if ($cgi->validateFailures() &&
									 $cgi->internalparam ("ERROR_SPKAC"));
	my $req = $cgi->validate ("reqEntry", \&IDX::PKI::ValidateInput::nakedpem,
							  "-default" => undef);
	$cgi->error (REQUEST_CREATE) if ($cgi->validateFailures() &&
									 $cgi->internalparam ("ERROR_reqEntry"));
	if ($spkac) {			# SPKAC
		$csr = new IDX::PKI::Crypto::CSR::SPKAC (-dn => \@dn,
												 -csr => "$spkac\n",
												 -form => \%forms);
	} elsif ($req) {		# PKCS#10
		$cgi->error (REQUEST_CREATE) if ($req eq "noreqEntry");

		$csr = new IDX::PKI::Crypto::CSR::PKCS10 (-dn => \@dn,
												  -csr => $req,
												  -form => \%forms);

		unless (IDX::PKI::DN::compare([$csr->getDN()], \@dn)) {
			$log->alert("Security Breach Attempt",
						-details => ["DN in PKCS#10 doesn't match request",
									 "Requested DN: ".IDX::PKI::DN::rfc2253
									 (@dn),
									 "PKCS#10 DN: ".IDX::PKI::DN::rfc2253
									 ($csr->getDN())],
						-attachment => $csr);
			$cgi->error (REQUEST_CREATE);
		}
	} else {	# No request
		$cgi->error (REQUEST_CREATE);
	}
} elsif ($type eq "central") {
	$csr = new IDX::PKI::Crypto::CSR::Central (-dn => \@dn,
											   -csr => "Central",
											   -form => \%forms);
} elsif ($type eq "server") {
	my $pkcs10 = delete ($forms{'pkcsten'});
	@dn = parse IDX::PKI::Crypto::PKCS10 ($pkcs10)->getDN();
	# Server requests are a bit special: We first go to an intermediate
	# page displaying the values, and only then we send the CSR.
	if ($cgi->validate ("sendrequest", \&IDX::PKI::ValidateInput::boolean,
						"-default" => undef)) {
		# Confirmation OK, send the CSR.
		$csr = new IDX::PKI::Crypto::CSR::PKCS10 (-dn => \@dn,
												  -csr => $pkcs10,
												  -form => \%forms);
	} elsif ($cgi->validate ("editrequest", \&IDX::PKI::ValidateInput::boolean,
							 "-default" => undef)) {
		# Confirmation not OK, return to the form (but this is not an
		# error or a failure, so we use a specific view).
		$cgi->terminate ("form", 0);
	} else {
		# Ask for confirmation.
		my %data = (
					# Some fields we know what to do with...
					$forms{"subjectAltName"} ?
					("subjectAltName" => $forms{"subjectAltName"}) : (),
					$forms{"adminEmail"} ? 
					("adminEmail" => $forms{"adminEmail"}) : (),
					);
		my @dncopy = @dn;
		my @dnforphp;
		while (my ($short, $val) = splice (@dncopy, 0, 2)) {
			my $long = IDX::PKI::DN::short_to_long ($short);
			push (@dnforphp, $long, $val) if ($val && $long);
		}
		$cgi->terminate ("confirm", [\@dnforphp, \%data]);
	}
} else {
	throw IDX::PKI::Error::Internal ("Unknown request type '$type'");
}

$cgi->failure (REQUEST_CREATE) unless ($csr);

# Key size verification.
if (($type ne "central") &&
	$csr->getAttribute ("pk_bitsize") < $profile->getKeySize()) {
	$log->failure("EE_CSR",
				  -details => "Key size was too small",
				  -attachment => $csr);
	$cgi->error (INPUT_KEYSIZE);
}

# Add Request Identifier
my $hasher = new IDX::PKI::Crypto::Hash();
my $tracking = $hasher->hash($csr->serialize().gmtime());
$csr->setFormEntry("tracking", $tracking);
$log->addTracking($tracking);

################ Sending the CSR ###############################
try {
	$log->attempt
		({-subject => "CSR_CREATE",
		  -attachment => $csr},
		 sub {
			 $transport->send ($csr);
		 });
} catch IDX::PKI::Error::Network with {
	my ($E) = @_;
	$log->alert ("CSR_SENT", "-details" => $E);
	$cgi->failure (TRANSPORT_SENDING);
};

################ Email notification ###############################

my $username = $csr->getSubject ();
my $useraddr = $csr->getEmail ();
my $destination = $administrator;


my $message = << "EOF";
[French translation at the end. / Traduction en franais  la fin.]

A new certificate request has been sent for $username ($useraddr).

This request has the tracking id: '$tracking'.

          ----------------------------------------

Une nouvelle demande de certificat a t envoye pour $username
($useraddr).

Cette requte a pour identifiant de suivi: '$tracking'.

EOF

try {
	sendmail ($administrator, $destination,
			  "Certificate Request Notification / Notification de demande ".
			  "de certificat", $message);
} catch IDX::PKI::Error with {
	my ($E) = @_;
	$log->alert("Error",
				-details => "Couldn't send an email to the administrator",
				-attachment => $E);
};
$cgi->success (0);
