#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Crypto::Hash;
use IDX::PKI::Crypto::CRR;
use IDX::PKI::Transport;

use MIME::Base64;

=pod "

=head1 NAME

B<ee-recover.cgi> - Sends a CRR to the RA

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews ("ee-recover.php");
$cgi->authorize ("__NONE__");

# No submit parameter: Ask informations to the user.
$cgi->success (undef)
	unless ($cgi->validate ("submit", \&IDX::PKI::ValidateInput::boolean));

############### Configuration informations ####################
my $section = $config->getval ('EE', 'krr_transport_out');
my $transport = new IDX::PKI::Transport::Sender
	("TRANSPORT-OUT-".$section);

############### Call parameters ####################

my $serial = $cgi->validate ("idxpkiSerial",
							 \&IDX::PKI::ValidateInput::serial);
if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_idxpkiSerial")) {
	$cgi->error (INPUT_HIDDEN);
}

# Identifies the request author, if we can know it.
if (my $peer = $cgi->getPeerCertificate()) {
	my $x509 = parse IDX::PKI::Crypto::X509 ($peer);
	$log->addAuthorizer ($x509);
}

# Identity of the requester (mandatory).
my $requester = $cgi->validate ("requester", \&IDX::PKI::ValidateInput::text);
if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_requester")) {
	$cgi->internalparam ("ERROR", "requester");
	($cgi->internalparam ("ERROR_requester") eq "MISSING") ?
		$cgi->error (INPUT_MISSING) : $cgi->error (INPUT_FORMAT);
}

# Revocation reason (mandatory).
my @reasons = ("unspecified", "keyLost", "serviceContinuity");
my $reason = $cgi->validate ("reason", \&IDX::PKI::ValidateInput::choice,
							 "-extraargs" => \@reasons);
if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_reason")) {
	$cgi->internalparam ("ERROR", "reason");
	($cgi->internalparam ("ERROR_reason") eq "MISSING") ?
		$cgi->error (INPUT_MISSING) : $cgi->error (INPUT_FORMAT);
}

# Free comment (optional).
my $comment = $cgi->validate ("comment",
							  \&IDX::PKI::ValidateInput::multilinetext,
							  "-extraargs" => 0, "-default" => undef);
if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_comment")) {
	$cgi->internalparam ("ERROR", "comment");
	$cgi->error (INPUT_FORMAT);
}

# Sends the KRR.
$log->attempt
	("KRR_SENT",
	 sub {
		 my $request = new
			 IDX::PKI::Message::TrustedRequest ('RECOVERY_REQUEST');

		 my $reader = new IDX::PKI::CertDB::CA ("EE");
		 my $cert = $reader->searchCerts ("serial" => $serial) ||
			 $cgi->error (REPOSITORY_NOENTRY);
		 my $profile = $reader->getCertAttr ($cert, "profile");

		 $request->addDelta (
							 'serial'          => $serial,
							 'idxpkiSubjectID' => $cert->getUserID (),
							 'idxpkiProfile'   => $profile,
							 #'encrypt_cert'    => $admin->serialize(),
							 "requester"       => $requester,
							 "reason"          => $reason,
							 "comment"         => encode_base64 ($comment, ""),
							 #'email'           => $admin->getEmail(),
							 );

		 # Add request identifier.
		 my $hasher = new IDX::PKI::Crypto::Hash();
		 my $tracking = $hasher->hash ($request->serialize().gmtime());
		 $request->addDelta ("tracking", $tracking);
		 $log->addTracking ($tracking);

		 # Sign the request
		 my $x509 = load IDX::PKI::Crypto::X509
			 ($config->getval ('EE', 'certificate'));
		 my $key = load IDX::PKI::Crypto::PrivateKey
			 ($config->getval ('EE', 'key'));
		 $request->sign ($key, $x509);

		 $transport->send ($request);
		 return {"-attachment" => $request};
	 });
$cgi->success (0);
