#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Crypto::Hash;
use IDX::PKI::Crypto::CRR;
use IDX::PKI::Transport;

use MIME::Base64;

=pod "

=head1 NAME

B<ee-revoke.cgi> - Search a certificate in ldap repository, creates a CRR and sends it to the RA

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews ("ee-revoke.php");
$cgi->authorize ("__NONE__");

# No submit parameter: Ask informations to the user.
$cgi->success (undef)
	unless ($cgi->validate ("submit", \&IDX::PKI::ValidateInput::boolean));

############### Configuration informations ####################
my $section = $config->getval ('EE', 'crr_transport_out') ||
	$cgi->failure (CONFIG_INVALID);
my $administrator = $config->getval ('EE', 'admin_email') ||
	$cgi->failure (CONFIG_INVALID);
my $transport = new IDX::PKI::Transport::Sender
	("TRANSPORT-OUT-".$section);

############### Call parameters ####################
# Empty return means the user has to validate.
my $serial = $cgi->validate ("idxpkiSerial",
							 \&IDX::PKI::ValidateInput::serial);
if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_idxpkiSerial")) {
	$cgi->error (INPUT_HIDDEN);
}

# Identifies the request author, if we can know it.
my $peer = $cgi->getPeerCertificate();
if ($peer) {
	my $x509 = parse IDX::PKI::Crypto::X509 ($peer);
	$log->addAuthorizer ($x509);
}

# Identity of the requester (mandatory).
my $requester = $cgi->validate ("requester", \&IDX::PKI::ValidateInput::text);
if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_requester")) {
	$cgi->internalparam ("ERROR", "requester");
	($cgi->internalparam ("ERROR_requester") eq "MISSING") ?
		$cgi->error (INPUT_MISSING) : $cgi->error (INPUT_FORMAT);
}

# Recovery reason (mandatory).
my @reasons = ("unspecified", "keyCompromise", "cACompromise",
			   "affiliationChanged", "cessationOfOperation",
			   "superseded", "certificateHold", "removeFromCRL");
my $reason = $cgi->validate ("reason", \&IDX::PKI::ValidateInput::choice,
							 "-extraargs" => \@reasons);
if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_reason")) {
	$cgi->internalparam ("ERROR", "reason");
	($cgi->internalparam ("ERROR_reason") eq "MISSING") ?
		$cgi->error (INPUT_MISSING) : $cgi->error (INPUT_FORMAT);
}

# Free comment (optional).
my $comment = $cgi->validate ("comment",
							  \&IDX::PKI::ValidateInput::multilinetext,
							  "-extraargs" => 0, "-default" => undef);
if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_comment")) {
	$cgi->internalparam ("ERROR", "comment");
	$cgi->error (INPUT_FORMAT);
}

# Sends the CRR.
$log->attempt
	("CRR_CREATE",
	 sub {
		 my $reader = new IDX::PKI::CertDB::CA ("EE");
		 my $cert = $reader->searchCerts ("serial" => $serial) ||
			 $cgi->error (REPOSITORY_NOENTRY);

		 my $crr = new IDX::PKI::Crypto::CRR ("-cert" => $cert->serialize());

		 # Add request identifier.
		 my $hasher = new IDX::PKI::Crypto::Hash();
		 my $tracking = $hasher->hash ($crr->serialize().gmtime());
		 $crr->setFormEntry ("tracking", $tracking);
		 $log->addTracking ($tracking);

		 my $profile = $reader->getCertAttr ($cert, "profile");
		 $crr->setFormEntry ("profile", $profile);

		 $crr->setFormEntry ("self", 1)
			 if ($peer && ($cert->serialize() eq $peer));

		 $crr->setFormEntry ("requester", $requester);
		 $crr->setFormEntry ("reason", $reason) if ($reason);
		 $crr->setFormEntry ("comment", encode_base64 ($comment, ""))
			 if ($comment);

		 $transport->send ($crr);
		 my $userdn = $cert->getSubjectDN();
		 my $certserial = $cert->getSerial();
		 my $destination = $administrator;

		 my $message = << "EOF";
[French translation at the end. / Traduction en franais  la fin.]

A revocation request has been sent for the certificate with serial number
$certserial (corresponding DN: $userdn).

          ----------------------------------------

Une demande de rvocation a t envoye pour le certificat de numro de
srie $certserial (DN associ: $userdn).
EOF

		IDX::PKI::Principal::sendmail
			($administrator, $destination,
			 "Revocation Request Notification / Notification de demande ".
			 "de revocation", $message);
		return {"-attachment" => [ $cert, $crr ],
				(($comment) ? ("-details" => $comment) : ())};
	});
$cgi->success (0);
