#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Crypto::CRR;
use IDX::PKI::Transport;
use IDX::PKI::CertDB;
use IDX::PKI::Debug(0);

=pod "

=head1 NAME

B<ra-crr-create.cgi> - Search a certificate in ldap repository, creates a CRR and sends it to the RA

=head1 PARAMETERS

The following CGI parameters are expected:

=item *

C<idxpkiSerial>: Serial number of the certificate to revoke.

=back

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews ("ra-crr-create.php");

############### Access Restriction ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Configuration informations ####################
my $section = $config->getval ('RA', 'crr_transport_out');
my $transport = new IDX::PKI::Transport::Sender
	("TRANSPORT-OUT-".$section);

############### Call parameters ####################
my $query = $cgi->validate ("idxpkiSerial",
							\&IDX::PKI::ValidateInput::serial);
if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_idxpkiSerial")) {
	$cgi->error (INPUT_HIDDEN);
}

my ($entry, $profile);
try {
	my $reader = new IDX::PKI::CertDB::CA ("RA");
	$entry = $reader->searchCerts ('serial' => $query);
	$profile = $reader->getCertAttr ($entry, "profile");

	&DEBUG("[debug]: DN: ". $entry->getSubjectDN .
		   " profile: ". $reader->getCertAttr($entry,"profile") ."\n");

} catch IDX::PKI::Error::Network with {
	my ($E) = @_;
	$log->alert ("CRR_CREATE", "-details" => "serial: $query",
				 "-attachment" => $E);
	$cgi->failure (REPOSITORY_CONNECT);
};

$cgi->failure (REPOSITORY_NOENTRY) unless ($entry);


my $cert = $entry->serialize();
my $crr = new IDX::PKI::Crypto::CRR (-cert => $cert);
try {
	$log->attempt
		({"-subject" => "CRR_CREATE",
		  "-attachment" => [$cert, $cgi->getPeerCertificate()]},
		 sub {
			 $transport->send ($crr);
		 });
} otherwise {
	$cgi->failure (TRANSPORT_SENDING);
};
$cgi->success (0);
