#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Crypto::CRR;
use IDX::PKI::Transport;
use IDX::PKI::Principal qw (readCRR);

use Tie::IxHash;
use MIME::Base64;

=pod "

=head1 NAME

B<ca-crr-details.cgi> - Details of a CRR

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews ("ra-crr-details.php");

############### Access Restriction ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Configuration informations ####################
my $section = $config->getval ("RA","crr_transport_in");
my $transport = new IDX::PKI::Transport::Unspooler
	("TRANSPORT-IN-".$section);

################ Read pending CRR ###############################
my $id = $cgi->validate ("id" => \&IDX::PKI::ValidateInput::messageID);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures());

my $crr;
try {
	$crr = readCRR ($transport, $id) || $cgi->failure (TRANSPORT_NOTFOUND);
} catch IDX::PKI::Error with {
	my ($E) = @_;
	$log->alert ("TRANSPORT_MSG",
				 "-details" => ["Transport: TRANSPORT-IN-".$section,
								"Message ID: $id"],
				 "-attachment" => $E);
	$cgi->failure (TRANSPORT_MSG);
};


my $x509 = parse IDX::PKI::Crypto::X509 ($crr->get_crr_openssl());

my %attr = %{$x509->get_attributes()};
my %ret;
tie (%ret, "Tie::IxHash");

$ret{"requester"} = $crr->getFormEntry ("requester");
$ret{"reason"}    = $crr->getFormEntry ("reason");
$ret{"comment"}   = decode_base64 ($crr->getFormEntry ("comment"))
	if ($crr->getFormEntry ("comment"));

foreach my $key (qw/not_before pk_bitsize not_after/) {
	$ret{$key} = $attr{$key};
}
$ret{"serial"}     = $x509->getSerial ();
$ret{'dn_subject'} = $x509->getSubjectDN ();
$ret{'dn_issuer'}  = $x509->getIssuerDN ();

$cgi->success (\%ret);
