#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Log;
use IDX::PKI::Error;
use IDX::PKI::Registry;
use IDX::PKI::Crypto::CRR;
use IDX::PKI::Crypto::X509;
use IDX::PKI::DN;
use IDX::PKI::Transport;
use IDX::PKI::Principal;
use IDX::PKI::Authentication qw(authorize is_authorized);

=pod "

=head1 NAME

B<ca-crr-list.cgi> - List pending CRRs

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews ("ra-crr-list.php");

############### Access Restriction ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Configuration informations ####################
my $transport = new IDX::PKI::Transport::Unspooler
  ("TRANSPORT-IN-".$config->getval("RA", "crr_transport_in"));

################ Read List of CRR #############################
my @list_of_messages;
try {
	@list_of_messages  = $transport->list();
} otherwise {
	$log->alert(TRANSPORT_MSG, -attachment => [ shift ]);
	$cgi->return(TRANSPORT_MSG);
};

my @pending_crr_list = ();

foreach my $id (@list_of_messages) {
	my $crr;
	try {
		$crr = readCRR ($transport, $id);
	} otherwise {
		$log->alert(TRANSPORT_MSG,
					-details => [ "Message ID: $id" ]
					-attachment => [ shift ]);
	};
	next unless ($crr);
	
		my $x509 = parse IDX::PKI::Crypto::X509 ($crr->get_crr_openssl() );
		my $dn = build IDX::PKI::DN ($x509->getSubjectDN ());
		push (@pending_crr_list, ([$id, $dn->getUserID(),
								   $crr->getFormEntry ("reason")]));
}

$cgi->success(\@pending_crr_list);
