#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Error;
use IDX::PKI::Registry;
use IDX::PKI::Profile;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Transport;
use IDX::PKI::Authentication qw(authorize is_authorized);
use IDX::PKI::Principal qw (readCSR);

=pod "

=head1 NAME

B<ra-csr-list.cgi> - List pending CSRs on the RA

=cut "

############### CGI, Config and Log init ####################
$cgi->setupViews ("ra-csr-list.php");

############### Access Restriction ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Configuration informations ####################
my $section = "TRANSPORT-IN-".$config->getval ("RA", "csr_transport_in");
my $transport = new IDX::PKI::Transport::Unspooler($section);

################ Read list of pending messages #############################
my @list_of_messages;
try {
	@list_of_messages = $transport->list();
} otherwise {
	my ($E) = @_;
	$log->alert ("TRANSPORT_MSG",
				 "-details" => "Transport: ".$section,
				 "-attachment" => $E);
	$cgi->failure (TRANSPORT_MSG);
};

my @pending_csr_list = ();

foreach my $id (@list_of_messages) {
	my $csr;
	try {
		$csr = readCSR ($transport, $id);
	} otherwise {
		my ($E) = @_;
		$log->alert ("TRANSPORT_MSG",
					 "-details" => ["Transport: ".$section,
									"Message ID: $id"],
					 "-attachment" => $E);
	};
	next unless ($csr);

		my $subject = $csr->getFormEntry ("subjectAltName");
		unless (defined $subject) {
			require IDX::PKI::DN;
			my $dn = build IDX::PKI::DN ($csr->getDN ());
			$subject = $dn->getUserID ();
		}
		push (@pending_csr_list, ([$id, $subject]));
}

$cgi->success (\@pending_csr_list);
