#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Log;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Transport;
use IDX::PKI::Principal qw (readCSR pullCSR);

=pod "

=head1 NAME

B<ra-csr-reject.cgi> - Reject a pending CSR

=cut "

############### CGI init ####################
$cgi->setupViews ("ra-csr-reject.php");

############### Access Restriction ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Configuration informations ####################
my $transport = new IDX::PKI::Transport::Unspooler
	("TRANSPORT-IN-".$config->getval ("RA", "csr_transport_in"));

################ Call Parameters ###############################
my $id = $cgi->validate ("id" => \&IDX::PKI::ValidateInput::messageID);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures () &&
							   $cgi->internalparam ("ERROR_id"));

################ Read pending CSR ###############################
my $csr = readCSR($transport, $id) || $cgi->failure (TRANSPORT_NOTFOUND);
$log->addTracking ($csr->getFormEntry ("tracking"));

my $name = $csr->getFormEntry ("profile");

# No commentdone parameter: Ask informations to the user.
$cgi->success (undef)
	unless ($cgi->validate ("commentdone", \&IDX::PKI::ValidateInput::boolean,
							"-default" => undef));

# Sends an email to the user...
my $reason = $cgi->validate ("reason",
							 \&IDX::PKI::ValidateInput::multilinetext);
my $message = <<"EOF";
[French translation at the end. / Traduction en franais  la fin.]

Your certificate request has been rejected.
The reason was:
$reason

          ----------------------------------------

Votre demande de certificat a t rejete.
La raison en tait:
$reason

EOF

if ($cgi->validateFailures () && $cgi->internalparam ("ERROR_reason")) {
	$cgi->internalparam ("ERROR", "reason");
	$cgi->error (($cgi->internalparam ("ERROR_reason") eq "MISSING") ?
				 INPUT_MISSING : INPUT_FORMAT);
}
my $email = $csr->getEmail ();
my $admin = (parse IDX::PKI::Crypto::X509($cgi->getPeerCertificate()))->
	getEmail ();
$admin ||= $config->getval ("EE", "admin_email");
IDX::PKI::Principal::sendmail ($admin, $email, "CSR rejected / CSR rejetee",
							   $message) if ($email && $email =~ m/\@/);

$csr = pullCSR ($transport, $id) || $cgi->failure (TRANSPORT_MSG);

$log->attempt
	(["CSR_REJECT", "-attachment" => $csr],
	 sub {
		 $transport->expunge();
	 });

my $fileId = $csr->getFormEntry ("fileId");
if (($name eq "scep") && $fileId)   {
    my $scepdir = $config->getPath("paths", "scepdir")
               || $cgi->error (CONFIG_INVALID);

    my $ret = system("/bin/mv", "-f", "$scepdir/pending/$fileId.der",
                                 "$scepdir/pending/$fileId.info",
                                 "$scepdir/rejected");
    $ret = system("/bin/rm","-f",
                "$scepdir/pending/$fileId.sent");


    $cgi->error(REQUEST_INVALID)
	    if ((-f "$scepdir/pending/$fileId.der") ||
            (-f "$scepdir/pending/$fileId.info") ||
            (-f "$scepdir/pending/$fileId.sent"));
	}

$cgi->success (0);
