#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ra-csr-send.cgi> - Sends a certificate request according to a profile

=cut "

use strict;

use MIME::Base64;
use IDX::PKI::Registry;
use IDX::PKI::Log;
use IDX::PKI::Error;
use IDX::PKI::Config;
use IDX::PKI::Profile;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Crypto::Hash;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::PKCS7 qw(pkcs7Encrypt);
use IDX::PKI::Crypto::Tools;
use IDX::PKI::Transport;
use IDX::PKI::Authentication qw(authorize);
use IDX::PKI::Principal qw (set_profile_parameter transmit_profile);
use IDX::PKI::ValidateInput;
use IDX::PKI::DN;

############### CGI init ####################
$cgi->setupViews ("-success" => "ra-csr-send.php",
				  "-error"   => "ra-csr-form.php",
				  "-form"    => "ra-csr-form.php",
				  "-confirm" => "ra-csr-confirm.php");

############### Access Restriction ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Configuration informations ####################
my $rakey   = $config->getval ("RA","key");
my $racert  = $config->getval ("RA","certificate");

############### Call Parameters ####################
my $profilename = $cgi->validate
  ("profile", qr/^\s*([a-z0-9\-_.]+)\s*$/i, "-default" => undef)
  || $cgi->error(INPUT_HIDDEN);


################ Loading the profile ###############################
my $profile = new IDX::PKI::Profile ($profilename);
# Returns the profile to the PHP script.
do {
	my @shown_rdn;
	foreach my $rdn ($profile->getDNElements ()) {
		push (@shown_rdn, $rdn)
			unless ($profile->elementOptions ($rdn, "raonly"));
	}
	my @shown_form;
	foreach my $form ($profile->getFormElements ()) {
		push (@shown_form, $form)
			unless ($profile->elementOptions ($form, "raonly"));
	}
	transmit_profile ($cgi, $profile, \@shown_rdn, \@shown_form);
};

# Build elements we want to use for the request: DN elements.
my @dn = ();
my %counts = ();
foreach my $rdn ($profile-> getDNElements ()) {
	my $short = $profile->elementShortName ($rdn);
	my $long  = $profile->elementLongName ($rdn);
	my $req   = $profile->elementImportance ($rdn);
	$counts{$long} ++;
	
	if ($req eq "fixed") {
		# Fixed.
		push (@dn, $short, $profile->elementDefaultValue ($rdn));
	} else {
		my $type      = $profile->elementType ($rdn);
		my $maxlength = $profile->elementMaximumLength ($rdn);
		my $value     = $cgi->validate
		  ($long.$counts{$long} => \&IDX::PKI::ValidateInput::password);
		my $checked_value;
		
		# If the user must type the value twice for confirmation.
		if ($profile->elementOptions ($rdn, "verify")) {
			my $value_a = $cgi->validate
			  ($long.$counts{$long}."a" => \&IDX::PKI::ValidateInput::password);
			my $value_b = $cgi->validate
			  ($long.$counts{$long}."b" => \&IDX::PKI::ValidateInput::password);
			if ($value_a eq $value_b) {
				# The two values match.
				$value = $value_a;
			} else {
				# The two values don't match.
				$cgi->internalparam ("ERROR", $long);
				$cgi->error (INPUT_VERIFY);
			}
		}

		# Checks the value given by the user.
		if (defined $value && $value ne "") {
			if ($type eq "choice") {
				$checked_value = $wf_functions{'choice'}->
				  ($value, $profile->elementChoices ($rdn));
			} elsif (exists $wf_functions{$type}) {
				$checked_value = $wf_functions{$type}->
				  ($value, $maxlength);
			} else {
				throw IDX::PKI::Error::Internal
				  ("Unknown validation type '$type'");
			}

			unless (defined $checked_value) {
				# Invalid value.
				$cgi->internalparam ("ERROR", $long);
				$cgi->error (INPUT_FORMAT);
			}
		} elsif ($req eq "mandatory") {
			# Mandatory value not supplied.
			$cgi->internalparam ("ERROR", $long);
			$cgi->error (INPUT_MISSING);
		} else {
			next;
		}

		push (@dn, $short, $checked_value);
	}
}

# Build elements we want to use for the request: Form elements.
my %forms = ("profile" => $profile->getName ());
%counts = ();
foreach my $form ($profile-> getFormElements ()) {
	my $short     = $profile->elementShortName ($form);
	my $long      = $profile->elementLongName ($form);
	my $req       = $profile->elementImportance ($form);
	my $type      = $profile->elementType ($form);
	my $maxlength = $profile->elementMaximumLength ($form);
	$counts{$long} ++;
	my $value;
	if($type eq 'pem') {
		$value = $cgi->validate
		  ($long.$counts{$long} => \&IDX::PKI::ValidateInput::pem);
	} else {
		$value = $cgi->validate
		  ($long.$counts{$long} => \&IDX::PKI::ValidateInput::password);
	}
	my $checked_value;

	# If the user must type the value twice for confirmation.
	if ($profile->elementOptions ($form, "verify")) {
		my $value_a = $cgi->validate
		  ($long.$counts{$long}."a" => \&IDX::PKI::ValidateInput::password);
		my $value_b = $cgi->validate
			  ($long.$counts{$long}."b" => \&IDX::PKI::ValidateInput::password);
		if ($value_a eq $value_b) {
			# The two values match.
			$value = $value_a;
		} else {
			# The two values don't match.
			$cgi->internalparam ("ERROR", $long);
			$cgi->error (INPUT_VERIFY);
		}
	}

	# Checks the value given by the user.
	if (defined $value && $value ne "") {
		if ($type eq "choice") {
			$checked_value = $wf_functions{'choice'}->
			  ($value, $profile->elementChoices ($form));
		} elsif (exists $wf_functions{$type}) {
			$checked_value = $wf_functions{$type}->($value, $maxlength);
		} else {
			throw IDX::PKI::Error::Internal
			  ("Unknown validation type '$type'");
		}

		unless (defined $checked_value) {
			# Invalid value.
			$cgi->internalparam ("ERROR", $long);
			$cgi->error (INPUT_FORMAT);
		}
	} elsif ($req eq "mandatory") {
		# Mandatory value not supplied.
		$cgi->internalparam ("ERROR", $long);
		$cgi->error (INPUT_MISSING);
	} elsif ($req eq "fixed") {
		# Fixed.
		$checked_value = $profile->elementDefaultValue ($form);
	} else {
		next;
	}

	# Encrypt field if necessary.
	my $encrypt = $profile->elementOptions ($form, "encrypt");
	if ($encrypt) {
		$checked_value = pkcs7Encrypt
		  ($checked_value, $config->getval ("certificates", $encrypt));
	}

	# Can be a single value or array reference.
	if (exists ($forms{$short}) && ref ($forms{$short}) eq "ARRAY") {
		# Already an array ref.
		push (@{$forms{$short}}, $checked_value);
	} elsif (exists $forms{$short}) {
		# Already a text value => must become an array.
		$forms{$short} = [$forms{$short}, $checked_value];
	} else {
		# No need to have an array ref.
		$forms{$short} = $checked_value;
	}
}

# Creates the CSR.
my $csr;
my $type = $profile->getType ();

if ($type eq "central") {
	$csr = new IDX::PKI::Crypto::CSR::Central (-dn => \@dn,
											   -csr => "Central", -form => \%forms);
} elsif ($type eq "server") {
	my $pkcs10 = delete ($forms{'pkcsten'});
	@dn = parse IDX::PKI::Crypto::PKCS10 ($pkcs10)->getDN();
	# Server requests are a bit special: We first go to an intermediate
	# page displaying the values, and only then we send the CSR.
	if ($cgi->validate ("sendrequest", \&IDX::PKI::ValidateInput::boolean,
						"-default" => undef)) {
		# Confirmation OK, send the CSR.
		$csr = new IDX::PKI::Crypto::CSR::PKCS10 (-dn => \@dn,
												  -csr => $pkcs10,
												  -form => \%forms);
	} elsif ($cgi->validate ("editrequest", \&IDX::PKI::ValidateInput::boolean,
							 "-default" => undef)) {
		# Confirmation not OK, return to the form (but this is not an
		# error or a failure, so we use a specific view).
		$cgi->terminate ("form", 0);
	} else {
		# Ask for confirmation.
		my %data = (
					# Some fields we know what to do with...
					$forms{"subjectAltName"} ?
					("subjectAltName" => $forms{"subjectAltName"}) : (),
					$forms{"adminEmail"} ? 
					("adminEmail" => $forms{"adminEmail"}) : (),
					);
		my @dncopy = @dn;
		my @dnforphp;
		while (my ($short, $val) = splice (@dncopy, 0, 2)) {
			my $long = IDX::PKI::DN::short_to_long ($short);
			push (@dnforphp, $long, $val) if ($val && $long);
		}
		$cgi->terminate ("confirm", [\@dnforphp, \%data]);
	}
} else {
	throw IDX::PKI::Error::Internal
	  ("Unknown request type '$type'");
}

$cgi->error(REQUEST_CREATE) unless (defined $csr);

# Key size verification.
if ((($type ne "central") && ($type ne "auto")) &&
	$csr->getAttribute ("pk_bitsize") < $profile->getKeySize()) {
	$cgi->error (INPUT_KEYSIZE);
}

# Adds extensions.
foreach my $ext ($profile->getExtensions ()) {
	$csr->{'form'}{$ext} = 1;
}

$csr->{form}->{admin} = encode_base64($cgi->getPeerCertificate(), '');
my $hasher = new IDX::PKI::Crypto::Hash();
my $tracking = $hasher->hash($csr->serialize().gmtime());
$csr->setFormEntry("tracking", $tracking);
$log->addTracking($tracking);

my $rv = 0;

try {
		my $key = load IDX::PKI::Crypto::PrivateKey ($rakey);
		my $cert = load IDX::PKI::Crypto::X509 ($racert);
		my $signedblob = new IDX::PKI::Crypto::SignedBlob (-object => $csr);
		$signedblob->sign (-key=> $key, -cert=> $cert);
		
		my $section = $config->getPath ('RA', 'csr_transport_out') ||
		  $cgi->failure (CONFIG_INVALID);
		
		my $transport = new IDX::PKI::Transport::Sender
		  ("TRANSPORT-OUT-".$section);
		
		$log->attempt
		  ({ -subject => "CSR_CREATE", -attachment => [$csr] },
		   sub {
			   my $key = load IDX::PKI::Crypto::PrivateKey ($rakey);
			   my $cert = load IDX::PKI::Crypto::X509 ($racert);
			   my $signedblob = new IDX::PKI::Crypto::SignedBlob (-object => $csr);
			   $signedblob->sign (-key=> $key, -cert=> $cert);
			   $transport->send ($signedblob);
		   });
} otherwise {
	$cgi->failure (TRANSPORT_SENDING);
};

$cgi->success ($rv);
