#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ra-csr-validate.cgi> - Modifies or accepts a pending CSR

=cut "

use strict;

use MIME::Base64;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Log;
use IDX::PKI::Config;
use IDX::PKI::Profile;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Crypto::Tools;
use IDX::PKI::Crypto::PrivateKey;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::SignedBlob;
use IDX::PKI::Transport;
use IDX::PKI::Authentication qw(authorize);
use IDX::PKI::ValidateInput;
use IDX::PKI::Principal;

use Tie::IxHash;

$cgi->setupViews("ra-csr-validate.php");

############### Access Restriction ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Configuration informations ####################
my ($rakey, $racert, $sect_in, $sect_out) = map {$config->getPath("RA", $_)}
	qw(key certificate csr_transport_in csr_transport_out);

$cgi->failure (CONFIG_INVALID)
	unless ($sect_in && $sect_out && $rakey && $racert);

my $transin = new IDX::PKI::Transport::Unspooler ("TRANSPORT-IN-".$sect_in);

############## Parse arguments #######################################
my $id = $cgi->validate("id" => \&IDX::PKI::ValidateInput::messageID);
# Are we just showing the contents or sending it forward to the CA.
my $issending = $cgi->validate
	("validate", \&IDX::PKI::ValidateInput::boolean, -default => undef);

$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures());

################ Read pending CSR ###############################
my $csr;
if ($issending) {
	$csr = pullCSR ($transin, $id) || $cgi->failure (TRANSPORT_NOTFOUND);
} else {
	$csr = readCSR ($transin, $id) || $cgi->failure (TRANSPORT_NOTFOUND);
}

$log->addTracking ($csr->getFormEntry ("tracking"));

my $profname = $csr->getFormEntry ("profile");
my $profile = new IDX::PKI::Profile ($profname);

# Gets and stores the CSR values.
# DN.
my %csrrdns;
tie (%csrrdns, "Tie::IxHash");
{
	my @dn = $csr->getDN ();
	while (my ($short, $val) = splice (@dn, 0, 2)) {
		$csrrdns{$short} = [] unless (defined $csrrdns{$short});
		push (@{$csrrdns{$short}}, $val);
	}
}	# End of scope.

# Form elements.
my %csrform;
tie (%csrform, "Tie::IxHash");
{
	my %data = $csr->getForms ();
	while (my ($short, $listorval) = each (%data)) {
		$csrform{$short} = (ref ($listorval) eq "ARRAY") ? $listorval :
			[$listorval];
	}
}	# End of scope.

# Gets informations for the view, according to the profile.
my (@rdns, @form);

# Profile informations: Element short name (known by the CSR), element
# long name (known by the view), format and mandatory character if
# editable.

# This is just code factorisation because RDNs and form elements are so
# much similar.
sub _store_elem ($$$$) {
	my ($profile, $elem, $valuesref, $dataref) = @_;
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-values" => $valuesref)
		unless (ref ($valuesref) eq "HASH");
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-data" => $dataref)
		unless (ref ($dataref) eq "ARRAY");

	my $short = $profile->elementShortName ($elem);
	my $long  = $profile->elementLongName ($elem);
	$valuesref->{$short} = [] unless (defined $valuesref->{$short});

	# Hidden from the view, but still present in the CSR.
	if ($profile->elementOptions ($elem, "encrypt")) {
		shift (@{$valuesref->{$short}});
		next;
	}

	my %infos = (
				 "short" => $short,
				 "key"   => $long,
				 "val"   => shift (@{$valuesref->{$short}}),
				 );
	if ($profile->elementOptions ($elem, "edit")) {
		$infos{"edit"}      = $profile->elementType ($elem);
		$infos{"mandatory"} = $profile->elementImportance ($elem);
		if ($infos{"edit"} eq "choice") {
			# Only useful if 'edit' says this is a choice...
			$infos{"choices"} = [$profile->elementChoices ($elem)];
		} else {
			# Only useful otherwise: max length...
			$infos{"length"} = $profile->elementMaximumLength($elem);
		}
	}
	push (@$dataref, \%infos);

	return $dataref;
}

# DN.
foreach my $elem ($profile->getDNElements ()) {
	_store_elem ($profile, $elem, \%csrrdns, \@rdns);
}

# Form elements.
foreach my $elem ($profile->getFormElements ()) {
	_store_elem ($profile, $elem, \%csrform, \@form);
}

# Extra parameter that we have in the CSR but weren't captured by the
# profile. Display only -- no edition.
# Currently we do this only for DN elements because form elements may
# contain things we don't want to show. This allows to display server
# PKCS#10 RDNs automatically.

# In fact this is a bad idea to do it this way because such extra
# parameters always go at the end, which may not be what we want.

# DN.
unless(scalar(@rdns)) {
	foreach my $short (keys (%csrrdns)) {
		if ($csrrdns{$short} && @{$csrrdns{$short}}) {
			foreach my $value (@{$csrrdns{$short}}) {
				push (@rdns, {
							  "short" => $short,
							  "key"   => IDX::PKI::DN::short_to_long ($short),
				"val"   => $value,
							 });
			}
		}
	}
}

# Request contents. Structure of rdns and forms is an array of anonymous
# arrays:
# {"key" => $key, "value" => $value, "edit" => $edit,
# "mandatory" => $mandatory}
my %ret = (
		   "profile"  => {$profile->getTitlesI18N ()},
		   "tracking" => $csr->getFormEntry ("tracking"),
		   "rdns"     => \@rdns,
		   "form"     => \@form,
		   );

# Returns the parameters.
$cgi->bigparam ("CSR", \%ret);

if ($issending) {
	# Checks the modifications and sends the request.
	# Currently only form elements may be edited (not RDNs).

	my (%counts, %values, %update);

	foreach my $param (@form) {
		my $type  = $param->{"edit"};
		my $value = $param->{"val"};
		my $short = $param->{"short"};

		# If the parameter is editable, check it.
		if ($type) {
			my $long = $param->{"key"};
			$counts{$long} ++;
			$update{$short} = 1;

			my @extraparams = $param->{"mandatory"} eq "mandatory" ? () :
				("-default" => undef);
			my $function;
			if ($type eq "choice") {
				$function = $wf_functions{"choice"};
				push (@extraparams, "-extraargs" => $param->{"choices"});
			} elsif (exists $wf_functions{$type}) {
				$function = $wf_functions{$type};
				push (@extraparams, "-extraargs" => $param->{"length"});
			} else {
				throw
					IDX::PKI::Error::Internal ("CONFIG_INVALID", "-details" =>
											   "Unknown validation type",
											   "-type" => $type);
			}
			$value = $cgi->validate ("$long.$counts{$long}", $function,
									 @extraparams);

			# Validation failed.
			if ($cgi->validateFailures () &&
				$cgi->internalparam ("ERROR_$long.$counts{$long}")) {
				$cgi->internalparam ("ERROR", $long);
				$cgi->error
					(($cgi->internalparam ("ERROR_$long.$counts{$long}")
					  eq "MISSING") ? INPUT_MISSING : INPUT_FORMAT);
			}
		}

		# Push values into some hash, so as to be able to edit the CSR
		# later.
		if (exists ($values{$short}) && ref ($values{$short}) eq "ARRAY") {
			push (@{$values{$short}}, $value);
		} elsif (exists $values{$short}) {
			$values{$short} = [$values{$short}, $value];
		} else {
			$values{$short} = $value;
		}
	}

	# Now update the CSR accordingly.
	foreach my $param (@form) {
		my $short = $param->{"short"};
		$csr->{"form"}{$short} = $values{$short} if ($update{$short});
	}
	$csr->{"form"}{"admin"} = encode_base64 ($cgi->getPeerCertificate (), "");

	# Fill in extensions
	# Should this be moved to the CA?
	foreach my $ext ($profile->getExtensions()) {
		$csr->{"form"}{$ext} = 1;
	}

	$log->attempt
		(["CSR_VALIDATE", "-attachment" => $csr],
		 sub {
			 my ($transout, $message);
			 $message = new IDX::PKI::Crypto::SignedBlob ("-object" => $csr);
			 $message->sign
			   ("-key" => load IDX::PKI::Crypto::PrivateKey ($rakey),
				"-cert" => load IDX::PKI::Crypto::X509 ($racert));
			 $transout = new IDX::PKI::Transport::Sender
			   ("TRANSPORT-OUT-".$sect_out);
			 $transout->send ($message);
		 });
	$transin->expunge();
}

$cgi->success (0);
