#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ra-recovery-accept.cgi> - Accept a pending key recovery request

=cut "

use strict;

use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Principal;
use IDX::PKI::Authentication qw(authorize);
use IDX::PKI::Transport;
use IDX::PKI::Message::TrustedRequest;

############### CGI init ####################
$cgi->setupViews ("ra-recovery-accept.php");

############### Access Restriction ####################
$cgi->authorize("PRIV_USE", "ACL-".$config->getval('RA','web_acl'));

############### Configuration informations ####################
my $rakeyf = $config->getval ("RA","key");
my $racertf = $config->getval ("RA","certificate");
my $transportin = new IDX::PKI::Transport::Unspooler
  ("TRANSPORT-IN-".$config->getval ('RA', 'krr_transport_in'));
my $transportout = new IDX::PKI::Transport::Sender
  ("TRANSPORT-OUT-".$config->getval ('RA', 'krr_transport_out'));

############### Call Parameters ####################

my $id = $cgi->validate("id" => \&IDX::PKI::ValidateInput::messageID);

# Read the message.
my $request;
try {
	$request = $transportin->pullByID
	  ($id, 'IDX::PKI::Message::TrustedRequest');
} catch IDX::PKI::Error::Network with {
	$cgi->failure (TRANSPORT_MSG);
};
$cgi->error (TRANSPORT_MSG) unless ($request);

my $x509 = parse IDX::PKI::Crypto::X509($cgi->getPeerCertificate());
my $admin = $x509->getEmail() || $config->getval ("CA", "admin_email");


$request->addDelta ('email' => $admin, 'encrypt_cert' => $x509->serialize());

# Add informations to the request.
$request->addTracing ('Authorizer' => $cgi->getPeerCertificate ());

# Accept the request.
my $racert = load IDX::PKI::Crypto::X509 ($racertf);
my $rakey = load IDX::PKI::Crypto::PrivateKey ($rakeyf);
$request->sign ($rakey, $racert);

# Send the request.
try {
	$log->attempt
	  ({"-subject" => "KRR_SENT",
		"-details" => "Certificate with serial ".
		$request->getState (undef, 'serial')},
	   sub {
		   $transportout->send ($request);
	   });
} catch IDX::PKI::Error::Network with {
	$transportin->release ();
	$cgi->failure (TRANSPORT_SENDING);
};

$transportin->expunge ();
$cgi->success (0);
