#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ra-recovery-create.cgi> - Create a key recovery request

=cut "

use strict;

use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Principal;
use IDX::PKI::Authentication qw(authorize);
use IDX::PKI::Transport;
use IDX::PKI::Message::TrustedRequest;
use IDX::PKI::CertDB;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::Hash;
use IDX::PKI::Debug(0);

############### CGI init ####################
$cgi->setupViews ("ra-recovery-create.php");

############### Access Restriction ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Call Parameters ####################
# Search in the repository LDAP to know the associated profile.
my $serial = $cgi->validate("serial", qr/([a-f-A-F0-9]+)/) ||
  $cgi->error(INPUT_HIDDEN);

my $x509 = parse IDX::PKI::Crypto::X509($cgi->getPeerCertificate());
$log->addAuthorizer($x509);
my $admin = $x509->getEmail() || $config->getval ("CA", "admin_email");

my $reader;
my @entries;
try {
	$reader  = new IDX::PKI::CertDB::CA ("RA");
	@entries = $reader->searchCerts ("serial" => $serial);
} catch IDX::PKI::Error::Network with {
	$cgi->failure (REPOSITORY_CONNECT);
};
# We should only get exactly one answer.
$cgi->error (REPOSITORY_NOENTRY) unless (@entries == 1);

my $entry = $entries[0];
my $profile = $reader->getCertAttr ($entry, "profile");


# Some informations about the person who created the request and the
# email to which the reply has to be sent.
my $email = $cgi->validate("email", \&IDX::PKI::ValidateInput::email)
  || $admin;

# Build the request.
my $request = new IDX::PKI::Message::TrustedRequest ('RECOVERY_REQUEST');
$request->addDelta ('serial' => $serial,
					'idxpkiSubjectID' => $entry->getUserID (),
					(defined $profile ? ('idxpkiProfile' => $profile) :
					 ()),
					'email' => $email,
					'encrypt_cert' => $x509->serialize());
$request->addTracing ('Authorizer' => $x509);

# Add tracking
my $hasher = new IDX::PKI::Crypto::Hash();
my $tracking = $hasher->hash($request->serialize().gmtime());
$request->addDelta(tracking => $tracking);
$log->addTracking($tracking);

# Sign the request
my $x509 = load IDX::PKI::Crypto::X509
  ($config->getPath ('RA', 'certificate'));
my $key = load IDX::PKI::Crypto::PrivateKey
  ($config->getPath ('RA', 'key'));
$request->sign ($key, $x509);

# Check if we are allowed to validate a key recovery for the given
# profile.  If yes, the message will be sent directly to the escrow
# module, otherwise it will be sent to the RA to be validated.
my $sectref;

if ($config->getPath ('RA', 'recovery_dovalidate') && defined($profile)) {
	try {
		$sectref = 'krr_transport_out';
	} catch IDX::PKI::Error::Security with {
		$sectref = 'krr_validation';
	};
} else {
	$sectref = 'krr_transport_out';
}

my $transport = new IDX::PKI::Transport::Sender
  ("TRANSPORT-OUT-".$config->getval ('RA', $sectref));

# Send the request.
try {
	$log->attempt
	  ({"-subject" => "KRR_SENT",
		"-details" => "certificate with serial ".
		$request->getState (undef, 'serial')},
	   sub {
		   $transport->send ($request);
	   });
} catch IDX::PKI::Error with {
	$cgi->failure (TRANSPORT_SENDING);
};

$cgi->success (0);
