#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ra-recovery-details.cgi> - Details of a pending key recovery request

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Transport;
use IDX::PKI::Message::TrustedRequest;
use IDX::PKI::CertDB;
use IDX::PKI::Principal qw (zulu_datenow);
use IDX::PKI::Authentication qw(authorize is_authorized);
use Tie::IxHash;
use MIME::Base64;

############### CGI, Config and Log init ####################
$cgi->setupViews ("ra-recovery-details.php");

############### Access Restriction ####################
$cgi->authorize ("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Configuration informations ####################
my $section = "TRANSPORT-IN-".$config->getval ('RA', 'krr_transport_in');
my $transport = new IDX::PKI::Transport::Unspooler($section);

################ Call Parameters #############################
my $id = $cgi->validate ("id" => \&IDX::PKI::ValidateInput::messageID);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures());

my $request;
try {
	$request = $transport->readByID ($id,
									 'IDX::PKI::Message::TrustedRequest');
} catch IDX::PKI::Error with {
	my ($E) = @_;
	$log->alert ("TRANSPORT_MSG",
				 "-details" => ["Transport: TRANSPORT-IN-".$section,
								"Message ID: $id"],
				 "-attachment" => $E);
	$cgi->failure (TRANSPORT_MSG);
};
$cgi->failure (TRANSPORT_MSG) unless ($request);


# Search in the repository LDAP to get infos on the certificate.
my %details;
tie (%details, "Tie::IxHash");
try {
	$details{"requester"} = $request->getState (undef, 'requester');
	$details{"reason"}    = $request->getState (undef, 'reason');
	$details{"comment"}   = decode_base64 ($request->getState (undef,
															 'comment'));
	my $reader = new IDX::PKI::CertDB::CA ("RA");
	my $cert = $reader->searchCerts
		("serial" => $request->getState (undef, 'serial'));
	$details{"idxpkiSerial"}       = $cert->getSerial();
	$details{"idxpkiSubjectID"}    = $cert->getUserID();
	$details{"idxpkiSubjectEmail"} = $cert->getEmail();
	$details{"profile"} = $request->getState (undef, 'idxpkiProfile');
	$details{"idxpkiCertExpirationTime"}  = $cert->getAttribute("not_after");
	$details{"idxpkiCertPublicationTime"} = $cert->getAttribute("not_before");
	my $revocation = $reader->getCertAttr($cert, "revocation");
	my $now = zulu_datenow();
	if ($revocation eq "V") {
		$details{"idxpkiStatus"} = "valid";
	} elsif ($revocation eq "R") {
		$details{"idxpkiStatus"} = "revoked";
	} elsif ($cert->getAttribute("not_after_zulu") < $now ||
			 $cert->getAttribute("not_before_zulu") > $now) {
		$details{"idxpkiStatus"} = "expired";
	} else {
		$details{"idxpkiStatus"} = "unknown";
	}
} catch IDX::PKI::Error::Network with {
	$cgi->failure (REPOSITORY_CONNECT);
};

$cgi->success ([$id, \%details]);
