#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: config-ca.cgi,v 1.29 2003/07/23 13:20:50 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perlscript/config/config-ca.cgi,v $

=pod "

=head1 NAME

B<config-ca.cgi> - Configuration of the Certification Authority

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Registry;

use constant DEFAULT_MODE => 'auto';
use constant DEFAULT_CENTRAL => 'profile';
use constant DEFAULT_ESCROW => 'profile';

$cgi->setupViews ("config-ca.php");

$cgi->authorize ("PRIV_CONFIG",
				 $config->getSectionName ("CA", ["ACL", "web_acl"]));

$log->addAuthorizer ($cgi->getPeerCertificate ());

############### Configuration informations ####################

# Get the mode from the configuration file
my $mode        = $config->getPath ('CA', 'mode') || DEFAULT_MODE;
my $central     = $config->getPath ('CA', 'central') || DEFAULT_CENTRAL;
my $admin_email = $config->getPath ('CA', 'admin_email');

# Check whether the validation button has been clicked.
my $valid = $cgi->validate ("valid", \&IDX::PKI::ValidateInput::boolean,
							"-default" => undef);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures());

if ($valid) {
	$mode = $cgi->validate ("mode", \&IDX::PKI::ValidateInput::choice,
							"-extraargs" => ["manual", "auto"]);
	$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures ());
	$central = $cgi->validate ("central", \&IDX::PKI::ValidateInput::choice,
							   "-extraargs" => ["user", "admin", "ee",
												"profile"]);
	$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures ());
	$admin_email = $cgi->validate ("admin_email",
								   \&IDX::PKI::ValidateInput::email);
	if ($cgi->validateFailures ()) {
		$cgi->internalparam ("ERROR", 'adminEmail');
		($cgi->internalparam ("ERROR_admin_email") eq "MISSING") ?
			$cgi->error (INPUT_MISSING) :
			$cgi->error (INPUT_FORMAT);
	}

	# Write the configuration on disk.
	try {
		$log->attempt
			(["CONFIG", -details => "Entity: CA"],
			 sub {
				 IDX::PKI::Config->updateLocked (sub {
					 my ($config) = @_;
					 $config->setval ('CA', 'mode',        $mode);
					 $config->setval ('CA', 'central',     $central);
					 $config->setval ('CA', 'admin_email', $admin_email);
				 });
				 return {-details => ["mode: \"$mode\"",
									  "central: \"$central\"",
									  "admin_email: \"$admin_email\""]};
			 });
	} catch IDX::PKI::Error with {
		$cgi->failure (CONFIG_SAVING);
	};
} else {
	$cgi->param ("mode",        $mode);
	$cgi->param ("central",     $central);
	$cgi->param ("admin_email", $admin_email);
}

$cgi->success ($valid ? 1 : 0);
