#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<config-certs-search.cgi> - Searches for certificates in the PKI repository LDAP server

=head1 DESCRIPTION

This scripts search user certificates.

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::CertDB;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Debug(0);
use IDX::PKI::Principal qw (zulu_datenow);

=pod "

=head1 PARAMETERS

At least one of the following parameters must be present:

=over 4

=item *

C<idxpkiSubjectID>: "User identifier" filter (see
L<IDX::PKI::Crypto::X509/getUserID> for what a "user identifier" is);

=item *

C<idxpkiSubjectEmail>: Email address filter;

=item *

C<idxpkiSerial>: Serial number filter.

=back

C<section> must be provided too, to be propagated to the next view.

=cut "

# We don't want change the API, but CADB require other variable name
my %compat = (
	"idxpkiSubjectID"    => "ID",
	"idxpkiSubjectEmail" => "email",
	"idxpkiSerial"       => "serial",
);

$cgi->setupViews ("config-certs-search.php");

# Authorize none first, so that we can check the proper section.
$cgi->authorize ("__NONE__");
my $section = $cgi->validate ("section", qr /^(ACL-[a-zA-Z]+)$/);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures () &&
							   $cgi->internalparam ("ERROR_section"));
$cgi->error (INPUT_HIDDEN) if (! $config->SectionExists ($section));
$cgi->authorize ("PRIV_CHPRIV", $section);
$log->addAuthorizer ($cgi->getPeerCertificate ());

################ Call parameters ###############################

# assign values passed by users
my %criteria = ();
foreach my $param (keys (%compat)) {
	my $value =
		$cgi->validate ($param, \&IDX::PKI::ValidateInput::LDAP_wildcard,
						"-default" => undef);
	if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_$param")) {
		$cgi->internalparam ("ERROR", $param);
		$cgi->error (INPUT_FORMAT);
	} else {
		$criteria{$compat{$param}} = $value if (defined $value);
	}
}
$cgi->error (INPUT_EMPTY) unless (%criteria);

################ Certificate search ###############################
my @certs;
my %different_dns;
try {
	my $reader = new IDX::PKI::CertDB::CA ("config");
	my $now = zulu_datenow();
	&DEBUG("[debug]: search criterion ". join(" ",%criteria)."\n");

	foreach my $cert ($reader->searchCerts (%criteria,
											"!extension" => "NSserver",
											"!extension" => "serverAuth")) {
		my %certValues;
		$certValues{"idxpkiSubject"} = $cert->getSubjectDN;
		$certValues{"idxpkiSubjectEmail"} = $cert->getEmail();

		unless ($different_dns{$certValues{'idxpkiSubject'}}) {
			$different_dns{$certValues{'idxpkiSubject'}} = 1;
			push (@certs, \%certValues);
			&DEBUG("[debug]: certificate found, info read in: ".
				   join(" ",%certValues)."\n");
		}
	}
} catch IDX::PKI::Error::Network with {
	$log->alert ("REPOSITORY_CONNECT", "-details" => shift);
	$cgi->failure (REPOSITORY_CONNECT);
};

# No more than $limit, if not null.
my $limit = $config->val ("RA", "max_search_count");
($limit && scalar (keys (%different_dns)) > $limit) ?
	$cgi->error (REPOSITORY_TOOMANY) : $cgi->success (\@certs);
