#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: config-renewal.cgi,v 1.12 2003/01/13 12:33:42 laura Exp $
# $Source: /opt/cvs/idxpki/sources/src/perlscript/config/config-renewal.cgi,v $

=pod "

=head1 NAME

B<config-renewal.cgi> - Configuration of the renewal module

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Registry;

$cgi->setupViews ("config-renewal.php");

$cgi->authorize ("PRIV_CONFIG",
				 $config->getSectionName ("renewal", ["ACL", "web_acl"]));

$log->addAuthorizer ($cgi->getPeerCertificate ());

# Check whether the validation button has been clicked.
my $valid = $cgi->validate ("valid", \&IDX::PKI::ValidateInput::boolean,
							"-default" => undef);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures());

if ($valid) {
	my @list;

	# Validate input.
	for (my $i = 0;
		 defined (my $days = $cgi->validate ("days_$i", qr/^0*(\d+)$/));
		 $i++) {
		if ($cgi->validateFailures ()) {
			$cgi->internalparam ("ERROR", 'days');
			($cgi->internalparam ("ERROR_days_$i") eq "MISSING") ?
				$cgi->error (INPUT_MISSING) :
				$cgi->error (INPUT_FORMAT);
		}
		# Much like IDX::PKI::ValidateInput::multilinetext but allows
		# special chars.
		my $allowed_chars = join ("", map {sprintf ("%s", chr())}
								  (32..126, 160..255));
		$allowed_chars .= "\r\n";
		my $text = $cgi->validate ("text_$i", qr /^([$allowed_chars]*)$/,
								   "-default" => "");
		if ($cgi->validateFailures ()) {
			$cgi->internalparam ("ERROR", 'message');
			$cgi->error (INPUT_FORMAT);
		}
		my $active = $cgi->validate ("active_$i",
									 \&IDX::PKI::ValidateInput::boolean,
									 "-default" => undef);
		push (@list, {"days" => $days, "active" => $active, "text" => $text});
	}

	# Write the configuration on disk.
	try {
		$log->attempt
			(["CONFIG", -details => "Entity: renewal"],
			 sub {
				 IDX::PKI::Config->updateLocked (sub {
					 my ($config) = @_;
					 $config->setval ("renewal", "days",
									  map {$_->{"days"}} @list);
					 $config->setval ("renewal", "active",
									  map {$_->{"active"}} @list);
					 # We have texts, the config wants files.
					 my @origfiles = $config->getval ("renewal", "textfile");
					 my $basepath  = $config->getval ("paths", "configdir");
					 my @files;
					 my %seenfiles;	# Avoids two messages into the same file!
					 for (my $i = 0; $i < scalar (@list); $i ++) {
						 # Filename: Take existing one or compute a new
						 # one.
						 my $file = (exists ($origfiles[$i]) &&
									 ! $seenfiles{$origfiles[$i]}) ?
							 $origfiles[$i] :
							 "$basepath/renewal_message-".($i + 1).".conf";
						 $file =~ m|^($basepath/[a-z0-9.\-_]+)$|i ||
							 throw IDX::PKI::Error::Config
							 ("Incorrect renewal file #$i",
							  "-details" => $file);
						 $files[$i] = $1;
						 $seenfiles{$1} = 1;

						 # We can't ensure that the writing of renewal
						 # message files is atomic with config
						 # modification. This isn't _that_ important
						 # anyway.
						 local *FILE;
						 open (FILE, ">", $files[$i]) || throw
							 IDX::PKI::Error::IO ("Can't write renewal text.",
												  "-IOfile" => $files[$i]);
						 print FILE $list[$i]{"text"};
						 close FILE;
					 }
					 $config->setval ("renewal", "textfile", @files);

				 });
				 return {"-details" => ["list: ", \@list]};
			 });
	} catch IDX::PKI::Error with {
		$cgi->failure (CONFIG_SAVING);
	};
} else {
	# Read configuration from disk
	my @days_list   = $config->getPath ("renewal", "days");
	my @active_list = $config->getPath ("renewal", "active");
	my @file_list   = $config->getPath ("renewal", "textfile");

	for (my $i = 0; exists ($file_list[$i]); $i++) {
		$cgi->param("days_$i", $days_list[$i]);
		$cgi->param("active_$i", $active_list[$i]);
		local *FILE;
		open (FILE, "<", $file_list[$i]) || throw IDX::PKI::Error::IO
			("Unable to open renewal text.", "-IOfile" => $file_list[$i]);
		my $text = join ("", <FILE>);
		close FILE;
		$cgi->param("text_$i", $text);
	}
}

$cgi->success (0);

# Save $text into $filename
#
# $text is a reference on the text to write

sub save_file {
	my ($filename, $text) = @_;

	throw IDX::PKI::Error::IO("Unable to create file $filename.")
		unless open (FILE, ">$filename");
	print FILE $$text;
	close (FILE);
}
