#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: listprivs.cgi,v 1.18 2003/01/23 16:45:24 laura Exp $
# $Source: /opt/cvs/idxpki/sources/src/perlscript/config/listprivs.cgi,v $

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Registry;

use IDX::PKI::Authentication qw (%privilegeNames);

=pod "

=head1 NAME

B<listprivs.cgi> - Returns current allowed administrators for a given section

=head1 SYNOPSIS

=cut "

$cgi->setupViews ("privs.php");

# Authentication section depends on an input parameter...
$cgi->authorize ("__NONE__");
my $section = $cgi->validate ("section", qr /^(ACL-[a-zA-Z]+)$/);
$cgi->error (INPUT_HIDDEN) if ($cgi->validateFailures () &&
							   $cgi->internalparam ("ERROR_section"));
$cgi->error (INPUT_HIDDEN) if (! $config->SectionExists ($section));

$cgi->authorize ("PRIV_CHPRIV", $section);

# Structure: Keys are admin DNs, and values are anonymous arrays of
# defined privileges (PRIV_*) for this admin.
my %admins;
foreach my $priv (keys %privilegeNames) {
	my $subsect = $privilegeNames{$priv};
	# Not getval() because may be undef.
	foreach my $admin ($config->val ($section, "can-$subsect")) {
		$admins{$admin} = [] unless (defined $admins{$admin});
		push (@{$admins{$admin}}, $priv);
	}
}

$cgi->success (\%admins);
