#! /usr/bin/perl -w
use strict;

#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.
#

# $Id: Check_translations.pl,v 1.3 2002/02/06 15:56:21 sbi Exp $ */
# $Source: /opt/cvs/idxpki/sources/src/php/i18n/Check_translations.pl,v $ */

#TODO: read the PHP files, recognize calls, and check they all exist
#  It will probably be difficult to check all defined messages are
#  called because of indirections and variables

my $DEBUG = 0;
my $LANGUAGES_DEFINITION_FILE = "lang.inc";
my $MSG = {};

sub file_read {
  my $filename = shift;
  my $res = "";
  local *FILE;
  open (FILE, $filename) || die "Cannot open $filename: $!\n";
  $res .= $_ while <FILE>;
  close FILE;
  return $res;
}

sub msg {
  my $tmp = shift;
  $tmp =~ s/\s*$/\n/s;
  print $tmp; 
}

sub info    { my $tmp = shift; &msg("   INFO: ".$tmp); }
sub error   { my $tmp = shift; &msg("  ERROR: ".$tmp); }
sub warning { my $tmp = shift; &msg("WARNING: ".$tmp); }

# Reading and checking the list of languages
#   Note: the syntax is rigid (double quotes, one line, ending comma)
my $List_of_languages = &file_read($LANGUAGES_DEFINITION_FILE);
my $Languages = {};

while ($List_of_languages =~ m/^\s*"(\w+)"\s*=>\s*"([^"]+)"\s*,\s*$/gm) {
  my ($language_code, $language_label) = ($1, $2);
  &error("language ``$language_code'' defined more than once!")
    if exists $Languages->{$language_code};
  $Languages->{$language_code} = $language_label;
  &info("found language ".&_lang($language_code));
}

# Finding the default language
#   Note: the syntax is rigid (double quotes, one line, ending semicolumn)
my $Default_language = "";
if ($List_of_languages =~ m/^\s*\$DEFAULT_LANGUAGE\s*=\s*"([^"]+)"\s*;\s*$/m) {
  my $Default_language_candidate = $1;
  if (exists $Languages->{$Default_language_candidate}) {
    $Default_language = $Default_language_candidate;
    &info("default language is ".&_lang($Default_language));
  } else {
    &error("default language ``$Default_language_candidate'' ".
           "not found in the list of languages!");
    exit 1;
  }
} else {
  &error("cannot find the default language!\n");
  exit 1;
};

sub _lang {
  my $lang = shift;
  if (exists $Languages->{$lang}) {
    return "``$lang'' (``$Languages->{$lang}'')";
  } else {
    return "``$lang'' (??? XXX ERROR NOT FOUND)";
  }
}

# Returns a false value if the directory cannot be found
sub parse_language {
  my $lang = shift;
  local *DIR; 
  unless (exists $Languages->{$lang}) {
    &error("cannot parse language ``$lang'': not in the list of languages");
    return 0;
  }
  opendir(DIR, $lang) || return 0;
  &info("Parsing language ".&_lang($lang));
  while (my $file = readdir(DIR)) {
    &info("  found file ``$lang/$file''");
    next unless $file =~ m/^(\w+)\.inc$/s;
    my $section_name = $1;
    my $full_file = "$lang/$file";
    my $content = &file_read($full_file);

    # Checking we find the right string definition
    if ($content =~
      m/^\s*\$MSG\["$section_name"\]\["$lang"\]\s*=\s*array\s*\(/m) {
      &info("    could find the right variable definition in it");
    } else {
      &error("    could not find the right variable definition in it");
      exit 1;
    }

    &info("    read its content: ".length($content)." bytes");
    while ($content =~ m/^\s*"([^"]+)"\s*=>/gm) {
      my $key = $1;
      if (exists $MSG->{$lang}->{$section_name}->{$key}) {
        &error("``$key'' defined several times in ``$lang/$section_name''");
        exit 1;
      }
      $MSG->{$lang}->{$section_name}->{$key} = 1;
      &info("        found key ``$key''");
    }
  }
  closedir(DIR);
  return 1;
}

# Parsing default language
if (&parse_language($Default_language)) {
  &info("Default language ".&_lang($Default_language)." successfully parsed");  
} else {
  &error("could not parse ".&_lang($Default_language)." -- check directory!");
  exit 1;
}

# Parsing all other languages
foreach my $lang (sort keys %{$Languages}) {
  next if $lang eq $Default_language;
  if (&parse_language($lang)) {
    &info("language ".&_lang($lang)." successfully parsed");  
  } else {
    &error("could not parse ".&_lang($lang)." -- check directory!");
  }
}

# Comparing languages to the default language

# Checking sections
foreach my $lang (sort keys %{$Languages}) {
  next if $lang eq $Default_language;
  foreach my $section (sort keys %{$MSG->{$lang}}) {
    unless (exists $MSG->{$Default_language}->{$section}) {
      &error("Section ``$section'' found in language ``$lang'', ".
             "not in default language ``$Default_language''");
    } 
  }
  foreach my $section (sort keys %{$MSG->{$Default_language}}) {
    unless (exists $MSG->{$lang}->{$section}) {
      &warning("Section ``$section'' found in default language ".
             "``$Default_language'', not in language ``$lang''");
    }
  }
}

# Checking fields
foreach my $lang (sort keys %{$Languages}) {
  next if $lang eq $Default_language;
  foreach my $section (sort keys %{$MSG->{$lang}}) {
    # the asymmetric case is dealt with above
    next unless exists $MSG->{$Default_language}->{$section} &&
                exists $MSG->{$lang}->{$section};
    foreach my $field (sort keys %{$MSG->{$lang}->{$section}}) {
      unless (exists $MSG->{$Default_language}->{$section}->{$field}) {
        &error("Field ``$section''/``$field'' found in language ``$lang'', ".
               "not in default language ``$Default_language''");
      } 
    }
    foreach my $field (sort keys %{$MSG->{$Default_language}->{$section}}) {
      unless (exists $MSG->{$lang}->{$section}->{$field}) {
        &warning("Field ``$section''/``$field'' found in default language ".
               "``$Default_language'', not in language ``$lang''");
      } 
    }
  }
}
