#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: csr-centralmode.pl,v 1.8 2003/01/07 17:06:47 laura Exp $
# $Source: /opt/cvs/idxpki/sources/src/test/integration/csr-centralmode.pl,v $

use strict;
use IDX::Test qw (:DEFAULT runCommand runWebRequest);
use IDX::PKI::FileManip;

number_of_tests (14);

#########################  Initialisation  #########################

my $port    = new IDX::Test::PortRange (1);
my $pidfile = new IDX::Test::Filename;
if ($ENV{'ADMIN_CERT'}) {		# HTTPS (stunnel/apache).
	require InstallLib unless (defined $InstallLib::topdir);
	my %CONFIG = InstallLib::parseConfigStatus ();
	my $stunnel = $CONFIG{'CONFIG_SBINDIR'}."/stunnel";
	my @command = ($stunnel, '-c', '-d', $port, '-r', $CONFIG{'CONFIG_PORT_HTTPS'},
		'-P', $pidfile);

	my $pems;
	# Private key and certificate to use
	$pems = new IDX::Test::Filename;
	open (PEMS, ">", $pems);
	open (KEY,  "<", $ENV{'ADMIN_KEY'}) ||
		die "Can't open the private key file.";
	while (my $line = <KEY>) {
		print PEMS $line;
	}
	close (KEY);
	print PEMS "\n";
	open (CERT, "<", $ENV{'ADMIN_CERT'}) ||
		die "Can't open the certificate file.";
	while (my $line = <CERT>) {
		print PEMS $line;
	}
	close (CERT);
	print PEMS "\n";
	close (PEMS);
	push (@command, "-p", $pems);

	system (@command) && die "Couldn't open the SSL tunnel: $?.";

	# Warning: Don't forget the "kill (15, `cat $pidfile`);" at the end of
	# the tests!
} else {						# HTTPS (stunnel/apache).
	# Good authentification on the secured entities.
	$ENV{'SSL_CLIENT_VERIFY'} = "SUCCESS";
	$ENV{'SSL_CLIENT_S_DN'}   = $ENV{'ADMIN_DN'};
}

sub launch_command {
	my ($script, @input) = @_;
	my $output;

	if ($ENV{'ADMIN_CERT'}) {	# HTTPS (stunnel/apache).
		$script .= "?" . join ("&", @input) if (@input);
		$output = &runWebRequest ($script, "http",
			'-host' => "localhost", '-port' => $port);
	} else {					# Command line.
		my @command = ("perl", "$ENV{'PERLSCRIPTSDIR'}/$script");
		$output = &runCommand (@command, @input);
	}

	$output =~ s/\r\n/\n/gs;

	my @output = split (/^$/m, $output);
	my @chomped_output;
	# XXX Yeck.  But is there a cleaner way to do it?
	foreach my $value (@output) {
		$value =~ s/\n*$//s;
		$value =~ s/^\n*//s;
		push (@chomped_output, $value);
	}

	return (wantarray ? @chomped_output : $output);
}

# We need the RA and CA to work in auto mode...
&launch_command ("config/config-ra.cgi",
	'valid=1',
	'mode=manual',
);
&launch_command ("config/config-ca.cgi",
	'valid=1',
	'mode=manual',
);

#########################  Normal Workflow  #########################

##### EE
# Create a request.
test "EE: valid central CSR" => sub {
	my @input = (
		'commonName=A User',
		'organizationName=IDX-PKI Testers',
		'emailAddress=test1@domain.net',
		'central=1',
	);
	my ($header, $vars, $data) = &launch_command ("EE/ee-csr-create.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data eq "0") {
		TESTFAIL ();
		print STDERR "Didn't get the expected return code: ".
			"Got '$data'.\n";
		return;
	}

	TESTOK ();
};

# Creates a second request.
&launch_command ("EE/ee-csr-create.cgi",
	'commonName=Another User',
	'organizationName=IDX-PKI Testers',
	'emailAddress=test2@domain.net',
	'central=1',
);

my $usertoaccept;
my $idtoaccept;
my $idtoreject;

##### RA
# List requests.
test "RA: list pending CSR" => sub {
	my ($header, $vars, $data) = &launch_command ("RA/ra-csr-list.cgi");

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	eval "\$data = $data;";

	unless (ref ($data) eq "ARRAY") {
		TESTFAIL ();
		print STDERR "The data does not look like an array: $data.\n";
		return;
	}

	unless (scalar (@$data) >= 2) {
		TESTFAIL ();
		print STDERR "There should be at least two requests.\n";
		return;
	}

	unless (ref ($data->[0]) eq "ARRAY") {
		TESTFAIL ();
		print STDERR "Unexpected data format: $data->[0].\n";
		return;
	}

	$usertoaccept = $data->[0][1];
	$idtoaccept   = $data->[0][0];
	$idtoreject   = $data->[1][0];

	TEST ($idtoaccept && $idtoreject);
};

# Request details.
test "RA: details of a pending CSR" => sub {
	my @input = (
		"id=".$idtoaccept,
	);
	my ($header, $vars, $data) = &launch_command ("RA/ra-csr-details.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	eval "\$data = $data;";

	unless (ref ($data) eq "HASH") {
		TESTFAIL ();
		print STDERR "The data does not look like a hash: $data.\n";
		return;
	}

	unless ($data->{'CN'} eq $usertoaccept) {
		TESTFAIL ();
		print STDERR "Can't recognise the user CN: $data->{'CN'}.\n";
		return;
	}

	TESTOK ();
};

# Accept request.
test "RA: accept a CSR" => sub {
	my @input = (
		"id=".$idtoaccept,
	);
	my ($header, $vars, $data) = &launch_command ("RA/ra-csr-accept.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data eq "0") {
		TESTFAIL ();
		print STDERR "Didn't get the expected return code: ".
			"Got '$data'.\n";
		return;
	}

	TESTOK ();
};

# Accepted request has disappeared.
test "RA: accepted CSR has disappeared" => sub {
	my @input = (
		"id=".$idtoaccept,
	);
	my ($header, $vars, $data) = &launch_command ("RA/ra-csr-details.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data eq "-33") {
		TESTFAIL ();
		print STDERR "Didn't get the expected return code: ".
			"Got '$data'.\n";
		return;
	}

	TESTOK ();
};

# Reject request.
test "RA: reject a CSR" => sub {
	my @input = (
		"id=".$idtoreject,
	);
	my ($header, $vars, $data) = &launch_command ("RA/ra-csr-reject.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data eq "0") {
		TESTFAIL ();
		print STDERR "Didn't get the expected return code: ".
			"Got '$data'.\n";
		return;
	}

	TESTOK ();
};

# Rejected request has disappeared.
test "RA: rejected CSR has disappeared" => sub {
	my @input = (
		"id=".$idtoreject,
	);
	my ($header, $vars, $data) = &launch_command ("RA/ra-csr-details.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data eq "-33") {
		TESTFAIL ();
		print STDERR "Didn't get the expected return code: ".
			"Got '$data'.\n";
		return;
	}

	TESTOK ();
};

# Create a request on the RA.
test "RA: valid central CSR" => sub {
	my @input = (
		'commonName=Yet Another User',
		'organizationName=IDX-PKI Testers',
		'emailAddress=test3@domain.net',
		'Central=1',
	);
	my ($header, $vars, $data) = &launch_command ("RA/ra-csr-request.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data eq "0") {
		TESTFAIL ();
		print STDERR "Didn't get the expected return code: ".
			"Got '$data'.\n";
		return;
	}

	TESTOK ();
};

##### CA
# List requests.
test "CA: list pending CSR" => sub {
	my ($header, $vars, $data) = &launch_command ("CA/ca-csr-list.cgi");

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	eval "\$data = $data;";

	unless (ref ($data) eq "ARRAY") {
		TESTFAIL ();
		print STDERR "The data does not look like an array: $data.\n";
		return;
	}

	unless (scalar (@$data) >= 2) {
		TESTFAIL ();
		print STDERR "There should be at least two requests.\n";
		return;
	}

	unless (ref ($data->[0]) eq "ARRAY") {
		TESTFAIL ();
		print STDERR "Unexpected data format: $data->[0].\n";
		return;
	}

	$usertoaccept = $data->[0][1];
	$idtoaccept   = $data->[0][0];
	$idtoreject   = $data->[1][0];

	TEST ($idtoaccept && $idtoreject);
};

# Request details.
test "CA: details of a pending CSR" => sub {
	my @input = (
		"id=".$idtoaccept,
	);
	my ($header, $vars, $data) = &launch_command ("CA/ca-csr-details.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	eval "\$data = $data;";

	unless (ref ($data) eq "ARRAY") {
		TESTFAIL ();
		print STDERR "The data does not look like an array: $data.\n";
		return;
	}

	my ($dn, $other) = @$data;

	unless ((scalar (@$data) == 2) && (ref ($dn) eq "ARRAY") &&
		(ref ($other) eq "HASH")) {
		TESTFAIL ();
		print STDERR "Unexpected data format: $dn -- $other.\n";
		return;
	}

	my %dn = @$dn;
	unless ($dn{'CN'} eq $usertoaccept) {
		TESTFAIL ();
		print STDERR "Can't recognise the user CN: $data->{'CN'}.\n";
		return;
	}

	TESTOK ();
};

# Accept request.
test "CA: accept a CSR" => sub {
	my @input = (
		"id=".$idtoaccept,
	);
	my ($header, $vars, $data) = &launch_command ("CA/ca-csr-accept.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data =~ m/\-\-\-\-\-BEGIN\ CERTIFICATE\-\-\-\-\-\n
			.+\n
			\-\-\-\-\-END\ CERTIFICATE\-\-\-\-\-/sx) {
		TESTFAIL ();
		print STDERR "The data does not look like a certificate:\n".
			"$data\n";
		return;
	}

	TESTOK ();
};

# Accepted request has disappeared..
test "CA: accepted CSR has disappeared" => sub {
	my @input = (
		"id=".$idtoaccept,
	);
	my ($header, $vars, $data) = &launch_command ("CA/ca-csr-details.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data eq "-22") {
		TESTFAIL ();
		print STDERR "Didn't get the expected return code: ".
			"Got '$data'.\n";
		return;
	}

	TESTOK ();
};

# Reject request.
test "CA: reject a CSR" => sub {
	my @input = (
		"id=".$idtoreject,
	);
	my ($header, $vars, $data) = &launch_command ("CA/ca-csr-reject.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data eq "0") {
		TESTFAIL ();
		print STDERR "Didn't get the expected return code: ".
			"Got '$data'.\n";
		return;
	}

	TESTOK ();
};

# Rejected request has disappeared..
test "CA: rejected CSR has disappeared" => sub {
	my @input = (
		"id=".$idtoreject,
	);
	my ($header, $vars, $data) = &launch_command ("CA/ca-csr-details.cgi",
		@input);

	unless ($header eq "Content-Type: text/plain") {
		TESTFAIL ();
		print STDERR "Didn't get the expected Content-Type header: ".
			"Got '$header'.\n";
		return;
	}

	unless ($data eq "-22") {
		TESTFAIL ();
		print STDERR "Didn't get the expected return code: ".
			"Got '$data'.\n";
		return;
	}

	TESTOK ();
};

#########################  Error Situations  #########################




#########################  End Cleaning  #########################

# Stop stunnel.
if ($ENV{'ADMIN_CERT'}) {		# HTTPS (stunnel/apache).
	kill (15, `cat $pidfile`);
}
